@@ -149,7 +149,7 @@ unsigned GidMapper::cellToGid(const Cell &cell) const
  */
 QByteArray GidMapper::encodeLayerData(const TileLayer &tileLayer,
                                       Map::LayerDataFormat format,
-                                      QRect bounds) const
+                                      QRect bounds, int compressionLevel) const
 {
     Q_ASSERT(format != Map::XML);
     Q_ASSERT(format != Map::CSV);
@@ -171,9 +171,11 @@ QByteArray GidMapper::encodeLayerData(const TileLayer &tileLayer,
     }
 
     if (format == Map::Base64Gzip)
-        tileData = compress(tileData, Gzip);
+        tileData = compress(tileData, Gzip, compressionLevel);
     else if (format == Map::Base64Zlib)
-        tileData = compress(tileData, Zlib);
+        tileData = compress(tileData, Zlib, compressionLevel);
+    else if (format == Map::Base64Zstandard)
+        tileData = compress(tileData, Zstandard, compressionLevel);
 
     return tileData.toBase64();
 }
@@ -189,8 +191,12 @@ GidMapper::DecodeError GidMapper::decodeLayerData(TileLayer &tileLayer,
     QByteArray decodedData = QByteArray::fromBase64(layerData);
     const int size = bounds.width() * bounds.height() * 4;
 
-    if (format == Map::Base64Gzip || format == Map::Base64Zlib)
-        decodedData = decompress(decodedData, size);
+    if (format == Map::Base64Gzip)
+        decodedData = decompress(decodedData, size, Gzip);
+    else if (format == Map::Base64Zlib)
+        decodedData = decompress(decodedData, size, Zlib);
+    else if (format == Map::Base64Zstandard)
+        decodedData = decompress(decodedData, size, Zstandard);
 
     if (size != decodedData.length())
         return CorruptLayerData;
