@@ -40,6 +40,7 @@ namespace Tiled {
 Layer::Layer(TypeFlag type, const QString &name, int x, int y) :
     Object(LayerType),
     mName(name),
+    mId(0),
     mLayerType(type),
     mX(x),
     mY(y),
@@ -51,6 +52,23 @@ Layer::Layer(TypeFlag type, const QString &name, int x, int y) :
 {
 }
 
+void Layer::resetIds()
+{
+    mId = 0;        // reset out own ID
+
+    switch (layerType()) {
+    case ObjectGroupType:
+        static_cast<ObjectGroup*>(this)->resetObjectIds();
+        break;
+    case GroupLayerType:
+        for (Layer *layer : static_cast<GroupLayer*>(this)->layers())
+            layer->resetIds();
+        break;
+    default:
+        break;
+    }
+}
+
 /**
  * Returns the effective opacity, which is the opacity multiplied by the
  * opacity of any parent layers.
@@ -147,11 +165,24 @@ QPointF Layer::totalOffset() const
     return offset;
 }
 
+/**
+ * Returns whether this layer can be merged down onto the layer below.
+ */
+bool Layer::canMergeDown() const
+{
+    const int index = siblingIndex();
+    if (index < 1)
+        return false;
+
+    Layer *lowerLayer = siblings().at(index - 1);
+    return lowerLayer->canMergeWith(this);
+}
+
 /**
  * A helper function for initializing the members of the given instance to
  * those of this layer. Used by subclasses when cloning.
  *
- * Layer name, position and size are not cloned, since they are assumed to have
+ * Layer name, position and size are not copied, since they are assumed to have
  * already been passed to the constructor. Also, map ownership is not cloned,
  * since the clone is not added to the map.
  *
@@ -160,6 +191,7 @@ QPointF Layer::totalOffset() const
  */
 Layer *Layer::initializeClone(Layer *clone) const
 {
+    clone->mId = mId;
     clone->mOffset = mOffset;
     clone->mOpacity = mOpacity;
     clone->mVisible = mVisible;
@@ -193,39 +225,41 @@ Layer *LayerIterator::next()
     Layer *layer = mCurrentLayer;
     int index = mSiblingIndex;
 
-    if (!layer) {
-        // Traverse to the first layer of the map
-        if (mMap && index == -1 && mMap->layerCount() > 0) {
-            layer = mMap->layerAt(0);
-            index = 0;
-        } else {
-            return nullptr;
-        }
-    } else {
+    do {
+        Q_ASSERT(!layer || (index >= 0 && index < layer->siblings().size()));
+
         // Traverse to next sibling
         ++index;
-    }
 
-    const auto siblings = layer->siblings();
-
-    // Traverse to parent layer if last child
-    if (index == siblings.size()) {
-        layer = layer->parentLayer();
-        index = layer ? layer->siblingIndex() : -1;
-    } else {
-        layer = siblings.at(index);
-
-        // If next layer is a group, traverse to its first child
-        while (layer->isGroupLayer()) {
-            auto groupLayer = static_cast<GroupLayer*>(layer);
-            if (groupLayer->layerCount() > 0) {
-                index = 0;
-                layer = groupLayer->layerAt(0);
-            } else {
+        if (!layer) {
+            // Traverse to the first layer of the map
+            if (mMap && index < mMap->layerCount())
+                layer = mMap->layerAt(index);
+            else
                 break;
+        }
+
+        const auto siblings = layer->siblings();
+
+        // Traverse to parent layer if last child
+        if (index == siblings.size()) {
+            layer = layer->parentLayer();
+            index = layer ? layer->siblingIndex() : mMap->layerCount();
+        } else {
+            layer = siblings.at(index);
+
+            // If next layer is a group, traverse to its first child
+            while (layer->isGroupLayer()) {
+                auto groupLayer = static_cast<GroupLayer*>(layer);
+                if (groupLayer->layerCount() > 0) {
+                    index = 0;
+                    layer = groupLayer->layerAt(0);
+                } else {
+                    break;
+                }
             }
         }
-    }
+    } while (layer && !(layer->layerType() & mLayerTypes));
 
     mCurrentLayer = layer;
     mSiblingIndex = index;
@@ -236,39 +270,45 @@ Layer *LayerIterator::next()
 Layer *LayerIterator::previous()
 {
     Layer *layer = mCurrentLayer;
-    int index = mSiblingIndex - 1;
+    int index = mSiblingIndex;
 
-    if (!layer) {
-        // Traverse to the last layer of the map if at the end
-        if (mMap && index < mMap->layerCount() && mMap->layerCount() > 0) {
-            layer = mMap->layerAt(index);
-        } else {
-            return nullptr;
-        }
-    } else {
-        // Traverse down to last child if applicable
-        if (layer->isGroupLayer()) {
-            auto groupLayer = static_cast<GroupLayer*>(layer);
-            if (groupLayer->layerCount() > 0) {
-                mSiblingIndex = groupLayer->layerCount() - 1;
-                mCurrentLayer = groupLayer->layerAt(mSiblingIndex);
-                return mCurrentLayer;
-            }
-        }
+    do {
+        Q_ASSERT(!layer || (index >= 0 && index < layer->siblings().size()));
 
-        // Traverse to previous sibling (possibly of a parent)
-        do {
-            if (index >= 0) {
-                const auto siblings = layer->siblings();
-                layer = siblings.at(index);
+        // Traverse to previous sibling
+        --index;
+
+        if (!layer) {
+            // Traverse to the last layer of the map if at the end
+            if (mMap && index >= 0 && index < mMap->layerCount())
+                layer = mMap->layerAt(index);
+            else
                 break;
+        } else {
+            // Traverse down to last child if applicable
+            if (layer->isGroupLayer()) {
+                auto groupLayer = static_cast<GroupLayer*>(layer);
+                if (groupLayer->layerCount() > 0) {
+                    index = groupLayer->layerCount() - 1;
+                    layer = groupLayer->layerAt(index);
+                    continue;
+                }
             }
 
-            layer = layer->parentLayer();
-            if (layer)
-                index = layer->siblingIndex() - 1;
-        } while (layer);
-    }
+            // Traverse to previous sibling (possibly of a parent)
+            do {
+                if (index >= 0) {
+                    const auto siblings = layer->siblings();
+                    layer = siblings.at(index);
+                    break;
+                }
+
+                layer = layer->parentLayer();
+                if (layer)
+                    index = layer->siblingIndex() - 1;
+            } while (layer);
+        }
+    } while (layer && !(layer->layerType() & mLayerTypes));
 
     mCurrentLayer = layer;
     mSiblingIndex = index;
@@ -285,7 +325,15 @@ void LayerIterator::toFront()
 void LayerIterator::toBack()
 {
     mCurrentLayer = nullptr;
-    mSiblingIndex = mMap ? mMap->layerCount() : -1;
+    mSiblingIndex = mMap ? mMap->layerCount() : 0;
+}
+
+bool LayerIterator::operator==(const LayerIterator &other) const
+{
+    return mMap == other.mMap &&
+            mCurrentLayer == other.mCurrentLayer &&
+            mSiblingIndex == other.mSiblingIndex &&
+            mLayerTypes == other.mLayerTypes;
 }
 
 
