@@ -51,6 +51,14 @@ class TileLayer;
  */
 class TILEDSHARED_EXPORT Layer : public Object
 {
+    Q_OBJECT
+
+    Q_PROPERTY(QString name READ name)
+    Q_PROPERTY(qreal opacity READ opacity)
+    Q_PROPERTY(bool visible READ isVisible)
+    Q_PROPERTY(bool locked READ isLocked)
+    Q_PROPERTY(QPointF offset READ offset)
+
 public:
     enum TypeFlag {
         TileLayerType   = 0x01,
@@ -66,6 +74,14 @@ public:
      */
     Layer(TypeFlag type, const QString &name, int x, int y);
 
+    /**
+     * The layer ID can be used to unique identify this layer of the map. It
+     * stays the same regardless of whether the layer is moved or renamed.
+     */
+    int id() const { return mId; }
+    void setId(int id) { mId = id; }
+    void resetIds();
+
     /**
      * Returns the type of this layer.
      */
@@ -84,20 +100,30 @@ public:
     /**
      * Returns the opacity of this layer.
      */
-    float opacity() const { return mOpacity; }
+    qreal opacity() const { return mOpacity; }
 
     /**
      * Sets the opacity of this layer.
      */
-    void setOpacity(float opacity) { mOpacity = opacity; }
+    void setOpacity(qreal opacity) { mOpacity = opacity; }
 
-    float effectiveOpacity() const;
+    qreal effectiveOpacity() const;
 
     /**
      * Returns the visibility of this layer.
      */
     bool isVisible() const { return mVisible; }
 
+    /**
+     * Returns the lock status of current layer.
+     */
+    bool isLocked() const { return mLocked; }
+
+    /**
+     * Returns the lock status of layer including parent layers.
+     */
+    bool isUnlocked() const;
+
     bool isHidden() const;
 
     /**
@@ -105,6 +131,8 @@ public:
      */
     void setVisible(bool visible) { mVisible = visible; }
 
+    void setLocked(bool locked) { mLocked = locked; }
+
     /**
      * Returns the map this layer is part of.
      */
@@ -156,6 +184,8 @@ public:
 
     QPointF totalOffset() const;
 
+    bool canMergeDown() const;
+
     virtual bool isEmpty() const = 0;
 
     /**
@@ -178,7 +208,7 @@ public:
     /**
      * Returns whether this layer can merge together with the \a other layer.
      */
-    virtual bool canMergeWith(Layer *other) const = 0;
+    virtual bool canMergeWith(const Layer *other) const = 0;
 
     /**
      * Returns a newly allocated layer that is the result of merging this layer
@@ -187,7 +217,7 @@ public:
      *
      * Should only be called when canMergeWith returns true.
      */
-    virtual Layer *mergedWith(Layer *other) const = 0;
+    virtual Layer *mergedWith(const Layer *other) const = 0;
 
     /**
      * Returns a duplicate of this layer. The caller is responsible for the
@@ -219,14 +249,16 @@ protected:
     Layer *initializeClone(Layer *clone) const;
 
     QString mName;
+    int mId;
     TypeFlag mLayerType;
     int mX;
     int mY;
     QPointF mOffset;
-    float mOpacity;
+    qreal mOpacity;
     bool mVisible;
     Map *mMap;
     GroupLayer *mParentLayer;
+    bool mLocked;
 
     friend class Map;
     friend class GroupLayer;
@@ -251,8 +283,9 @@ inline QPointF Layer::offset() const
 
 
 /**
- * An iterator for iterating over the layers of a map. When iterating forward,
- * group layers are traversed after their children.
+ * An iterator for iterating over the layers of a map, in the order in which
+ * they are drawn. When iterating forward, group layers are traversed after
+ * their children.
  *
  * Modifying the layer hierarchy while an iterator is active will lead to
  * undefined results!
@@ -260,7 +293,7 @@ inline QPointF Layer::offset() const
 class TILEDSHARED_EXPORT LayerIterator
 {
 public:
-    LayerIterator(const Map *map);
+    LayerIterator(const Map *map, int layerTypes = Layer::AnyLayerType);
     LayerIterator(Layer *start);
 
     Layer *currentLayer() const;
@@ -276,20 +309,30 @@ public:
     void toFront();
     void toBack();
 
+    // Allow use as general iterator and in range-based for loops
+    bool operator==(const LayerIterator &other) const;
+    bool operator!=(const LayerIterator &other) const;
+    LayerIterator &operator++();
+    LayerIterator operator++(int);
+    Layer *operator*() const;
+    Layer *operator->() const;
+
 private:
     const Map *mMap;
     Layer *mCurrentLayer;
     int mSiblingIndex;
+    int mLayerTypes;
 };
 
 
 /**
  * Iterate the given map, starting from the first layer.
  */
-inline LayerIterator::LayerIterator(const Map *map)
+inline LayerIterator::LayerIterator(const Map *map, int layerTypes)
     : mMap(map)
     , mCurrentLayer(nullptr)
     , mSiblingIndex(-1)
+    , mLayerTypes(layerTypes)
 {}
 
 /**
@@ -299,6 +342,7 @@ inline LayerIterator::LayerIterator(Layer *start)
     : mMap(start ? start->map() : nullptr)
     , mCurrentLayer(start)
     , mSiblingIndex(start ? start->siblingIndex() : -1)
+    , mLayerTypes(Layer::AnyLayerType)
 {}
 
 inline Layer *LayerIterator::currentLayer() const
@@ -329,6 +373,34 @@ inline bool LayerIterator::hasParent() const
     return mCurrentLayer && mCurrentLayer->parentLayer();
 }
 
+inline bool LayerIterator::operator!=(const LayerIterator &other) const
+{
+    return !(*this == other);
+}
+
+inline LayerIterator &LayerIterator::operator++()
+{
+    next();
+    return *this;
+}
+
+inline LayerIterator LayerIterator::operator++(int)
+{
+    LayerIterator it = *this;
+    next();
+    return it;
+}
+
+inline Layer *LayerIterator::operator*() const
+{
+    return mCurrentLayer;
+}
+
+inline Layer *LayerIterator::operator->() const
+{
+    return mCurrentLayer;
+}
+
 
 TILEDSHARED_EXPORT int globalIndex(Layer *layer);
 TILEDSHARED_EXPORT Layer *layerAtGlobalIndex(const Map *map, int index);
