@@ -37,15 +37,21 @@
 #include "tilelayer.h"
 #include "mapobject.h"
 
-#include <cmath>
+#include <QtMath>
 
 using namespace Tiled;
 
+Map::Map():
+    Map(Orthogonal, 0, 0, 0, 0, false)
+{
+}
+
 Map::Map(Orientation orientation,
          int width, int height, int tileWidth, int tileHeight, bool infinite):
     Object(MapType),
     mOrientation(orientation),
     mRenderOrder(RightDown),
+    mCompressionLevel(-1),
     mWidth(width),
     mHeight(height),
     mTileWidth(tileWidth),
@@ -54,36 +60,21 @@ Map::Map(Orientation orientation,
     mHexSideLength(0),
     mStaggerAxis(StaggerY),
     mStaggerIndex(StaggerOdd),
+    mChunkSize(CHUNK_SIZE, CHUNK_SIZE),
     mDrawMarginsDirty(true),
     mLayerDataFormat(Base64Zlib),
+    mNextLayerId(1),
     mNextObjectId(1)
 {
 }
 
-Map::Map(const Map &map):
-    Object(map),
-    mOrientation(map.mOrientation),
-    mRenderOrder(map.mRenderOrder),
-    mWidth(map.mWidth),
-    mHeight(map.mHeight),
-    mTileWidth(map.mTileWidth),
-    mTileHeight(map.mTileHeight),
-    mInfinite(map.mInfinite),
-    mHexSideLength(map.mHexSideLength),
-    mStaggerAxis(map.mStaggerAxis),
-    mStaggerIndex(map.mStaggerIndex),
-    mBackgroundColor(map.mBackgroundColor),
-    mDrawMargins(map.mDrawMargins),
-    mDrawMarginsDirty(map.mDrawMarginsDirty),
-    mTilesets(map.mTilesets),
-    mLayerDataFormat(map.mLayerDataFormat),
-    mNextObjectId(1)
+Map::Map(Orientation orientation,
+         QSize size, QSize tileSize, bool infinite)
+    : Map(orientation,
+          size.width(), size.height(),
+          tileSize.width(), tileSize.height(),
+          infinite)
 {
-    for (const Layer *layer : map.mLayers) {
-        Layer *clone = layer->clone();
-        clone->setMap(this);
-        mLayers.append(clone);
-    }
 }
 
 Map::~Map()
@@ -91,6 +82,44 @@ Map::~Map()
     qDeleteAll(mLayers);
 }
 
+void Map::setWidth(int width)
+{
+    if (width == mWidth)
+        return;
+
+    mWidth = width;
+    emit widthChanged();
+    emit sizeChanged();
+}
+
+void Map::setHeight(int height)
+{
+    if (height == mHeight)
+        return;
+
+    mHeight = height;
+    emit heightChanged();
+    emit sizeChanged();
+}
+
+void Map::setTileWidth(int width)
+{
+    if (width == mTileWidth)
+        return;
+
+    mTileWidth = width;
+    emit tileWidthChanged();
+}
+
+void Map::setTileHeight(int height)
+{
+    if (height == mTileHeight)
+        return;
+
+    mTileHeight = height;
+    emit tileHeightChanged();
+}
+
 QMargins Map::drawMargins() const
 {
     if (mDrawMarginsDirty)
@@ -116,12 +145,15 @@ QMargins Map::computeLayerOffsetMargins() const
 {
     QMargins offsetMargins;
 
-    for (const Layer *layer : mLayers) {
-        const QPointF offset = layer->offset();
-        offsetMargins = maxMargins(QMargins(std::ceil(-offset.x()),
-                                            std::ceil(-offset.y()),
-                                            std::ceil(offset.x()),
-                                            std::ceil(offset.y())),
+    for (const Layer *layer : allLayers()) {
+        if (layer->isGroupLayer())
+            continue;
+
+        const QPointF offset = layer->totalOffset();
+        offsetMargins = maxMargins(QMargins(qCeil(-offset.x()),
+                                            qCeil(-offset.y()),
+                                            qCeil(offset.x()),
+                                            qCeil(offset.y())),
                                    offsetMargins);
     }
 
@@ -164,60 +196,51 @@ void Map::recomputeDrawMargins() const
 int Map::layerCount(Layer::TypeFlag type) const
 {
     int count = 0;
-    LayerIterator iterator(this);
-    while (Layer *layer = iterator.next())
-       if (layer->layerType() == type)
-           count++;
+    LayerIterator iterator(this, type);
+    while (iterator.next())
+       count++;
     return count;
 }
 
-QList<ObjectGroup*> Map::objectGroups() const
-{
-    QList<ObjectGroup*> layers;
-    LayerIterator iterator(this);
-    while (Layer *layer = iterator.next())
-        if (ObjectGroup *og = layer->asObjectGroup())
-            layers.append(og);
-    return layers;
-}
-
-QList<TileLayer*> Map::tileLayers() const
-{
-    QList<TileLayer*> layers;
-    LayerIterator iterator(this);
-    while (Layer *layer = iterator.next())
-        if (TileLayer *tl = layer->asTileLayer())
-            layers.append(tl);
-    return layers;
-}
-
 void Map::addLayer(Layer *layer)
 {
-    adoptLayer(layer);
+    adoptLayer(*layer);
     mLayers.append(layer);
 }
 
-int Map::indexOfLayer(const QString &layerName, unsigned layertypes) const
+int Map::indexOfLayer(const QString &layerName, int layerTypes) const
 {
     for (int index = 0; index < mLayers.size(); index++)
         if (layerAt(index)->name() == layerName
-                && (layertypes & layerAt(index)->layerType()))
+                && (layerTypes & layerAt(index)->layerType()))
             return index;
 
     return -1;
 }
 
+Layer *Map::findLayer(const QString &name, int layerTypes) const
+{
+    LayerIterator it(this, layerTypes);
+    while (Layer *layer = it.next())
+        if (layer->name() == name)
+            return layer;
+    return nullptr;
+}
+
 void Map::insertLayer(int index, Layer *layer)
 {
-    adoptLayer(layer);
+    adoptLayer(*layer);
     mLayers.insert(index, layer);
 }
 
-void Map::adoptLayer(Layer *layer)
+void Map::adoptLayer(Layer &layer)
 {
-    layer->setMap(this);
+    if (layer.id() == 0)
+        layer.setId(takeNextLayerId());
 
-    if (ObjectGroup *group = layer->asObjectGroup())
+    layer.setMap(this);
+
+    if (ObjectGroup *group = layer.asObjectGroup())
         initializeObjectIds(*group);
 }
 
@@ -287,6 +310,18 @@ bool Map::replaceTileset(const SharedTileset &oldTileset,
     }
 }
 
+QSet<SharedTileset> Map::usedTilesets() const
+{
+    QSet<SharedTileset> tilesets;
+
+    // Only top-level layers need to be considered, since GroupLayer goes over
+    // its children
+    for (const Layer *layer : mLayers)
+        tilesets |= layer->usedTilesets();
+
+    return tilesets;
+}
+
 bool Map::isTilesetUsed(const Tileset *tileset) const
 {
     for (const Layer *layer : mLayers)
@@ -296,6 +331,33 @@ bool Map::isTilesetUsed(const Tileset *tileset) const
     return false;
 }
 
+std::unique_ptr<Map> Map::clone() const
+{
+    auto o = std::make_unique<Map>(mOrientation, mWidth, mHeight, mTileWidth, mTileHeight, mInfinite);
+    o->fileName = fileName;
+    o->exportFileName = exportFileName;
+    o->exportFormat = exportFormat;
+    o->mRenderOrder = mRenderOrder;
+    o->mHexSideLength = mHexSideLength;
+    o->mStaggerAxis = mStaggerAxis;
+    o->mStaggerIndex = mStaggerIndex;
+    o->mBackgroundColor = mBackgroundColor;
+    o->mChunkSize = mChunkSize;
+    o->mDrawMargins = mDrawMargins;
+    o->mDrawMarginsDirty = mDrawMarginsDirty;
+    for (const Layer *layer : mLayers) {
+        Layer *clone = layer->clone();
+        clone->setMap(o.get());
+        o->mLayers.append(clone);
+    }
+    o->mTilesets = mTilesets;
+    o->mLayerDataFormat = mLayerDataFormat;
+    o->mNextLayerId = mNextLayerId;
+    o->mNextObjectId = mNextObjectId;
+    o->setProperties(properties());
+    return o;
+}
+
 QList<MapObject*> Map::replaceObjectTemplate(const ObjectTemplate *oldObjectTemplate,
                                              const ObjectTemplate *newObjectTemplate)
 {
@@ -303,8 +365,8 @@ QList<MapObject*> Map::replaceObjectTemplate(const ObjectTemplate *oldObjectTemp
 
     QList<MapObject*> changedObjects;
 
-    for (auto group : objectGroups()) {
-        for (auto o : group->objects()){
+    for (auto layer : objectGroups()) {
+        for (auto o : static_cast<ObjectGroup*>(layer)->objects()) {
             if (o->objectTemplate() == oldObjectTemplate) {
                 o->setObjectTemplate(newObjectTemplate);
                 o->syncWithTemplate();
@@ -324,10 +386,38 @@ void Map::initializeObjectIds(ObjectGroup &objectGroup)
     }
 }
 
+Layer *Map::findLayerById(int layerId) const
+{
+    for (Layer *layer : allLayers()) {
+        if (layer->id() == layerId)
+            return layer;
+    }
+    return nullptr;
+}
+
+MapObject *Map::findObjectById(int objectId) const
+{
+    for (Layer *layer : objectGroups()) {
+        for (MapObject *mapObject : static_cast<ObjectGroup*>(layer)->objects()) {
+            if (mapObject->id() == objectId)
+                return mapObject;
+        }
+    }
+    return nullptr;
+}
+
+QRegion Map::tileRegion() const
+{
+    QRegion region;
+    LayerIterator it(this, Layer::TileLayerType);
+    while (auto tileLayer = static_cast<TileLayer*>(it.next()))
+        region |= tileLayer->region();
+    return region;
+}
+
 QString Tiled::staggerAxisToString(Map::StaggerAxis staggerAxis)
 {
     switch (staggerAxis) {
-    default:
     case Map::StaggerY:
         return QLatin1String("y");
     case Map::StaggerX:
@@ -347,7 +437,6 @@ Map::StaggerAxis Tiled::staggerAxisFromString(const QString &string)
 QString Tiled::staggerIndexToString(Map::StaggerIndex staggerIndex)
 {
     switch (staggerIndex) {
-    default:
     case Map::StaggerOdd:
         return QLatin1String("odd");
     case Map::StaggerEven:
@@ -367,7 +456,6 @@ Map::StaggerIndex Tiled::staggerIndexFromString(const QString &string)
 QString Tiled::orientationToString(Map::Orientation orientation)
 {
     switch (orientation) {
-    default:
     case Map::Unknown:
         return QLatin1String("unknown");
     case Map::Orthogonal:
@@ -397,10 +485,26 @@ Map::Orientation Tiled::orientationFromString(const QString &string)
     return orientation;
 }
 
+QString Tiled::compressionToString(Map::LayerDataFormat layerDataFormat)
+{
+    switch (layerDataFormat) {
+    case Map::XML:
+    case Map::Base64:
+    case Map::CSV:
+        return QString();
+    case Map::Base64Gzip:
+        return QLatin1String("gzip");
+    case Map::Base64Zlib:
+        return QLatin1String("zlib");
+    case Map::Base64Zstandard:
+        return QLatin1String("zstd");
+    }
+    return QString();
+}
+
 QString Tiled::renderOrderToString(Map::RenderOrder renderOrder)
 {
     switch (renderOrder) {
-    default:
     case Map::RightDown:
         return QLatin1String("right-down");
     case Map::RightUp:
