@@ -29,15 +29,22 @@
 #include "maprenderer.h"
 
 #include "imagelayer.h"
+#include "isometricrenderer.h"
+#include "map.h"
 #include "mapobject.h"
+#include "orthogonalrenderer.h"
 #include "tile.h"
 #include "tilelayer.h"
+#include "objectgroup.h"
 
 #include <QPaintEngine>
 #include <QPainter>
 #include <QVector2D>
 
+#include "qtcompat_p.h"
+
 #include <cmath>
+#include <memory>
 
 using namespace Tiled;
 
@@ -49,6 +56,30 @@ QRectF MapRenderer::boundingRect(const ImageLayer *imageLayer) const
     return QRectF(QPointF(), imageLayer->image().size());
 }
 
+QPainterPath MapRenderer::pointShape(const QPointF &position) const
+{
+    QPainterPath path;
+
+    const qreal radius = 10.0;
+    const qreal sweep = 235.0;
+    const qreal startAngle = 90.0 - sweep / 2;
+    QRectF rectangle(-radius, -radius, radius * 2, radius * 2);
+    path.moveTo(radius * cos(startAngle * M_PI / 180.0), -radius * sin(startAngle * M_PI / 180.0));
+    path.arcTo(rectangle, startAngle, sweep);
+    path.lineTo(0, 2 * radius);
+    path.closeSubpath();
+
+    QPainterPath hole;
+    const qreal smallRadius = radius / 2.0;
+    hole.addEllipse(QRectF(-smallRadius, -smallRadius, smallRadius * 2, smallRadius * 2));
+    path = path.subtracted(hole);
+
+    path.translate(pixelToScreenCoords(position) +
+                   QPointF(0, -2 * radius));
+
+    return path;
+}
+
 void MapRenderer::drawImageLayer(QPainter *painter,
                                  const ImageLayer *imageLayer,
                                  const QRectF &exposed)
@@ -102,10 +133,10 @@ void MapRenderer::drawPointObject(QPainter *painter, const QColor &color) const
     const QBrush opaqueBrush(color);
     painter->setBrush(opaqueBrush);
     const qreal smallRadius = radius / 3.0;
-    painter->drawEllipse(-smallRadius, -smallRadius, smallRadius * 2, smallRadius * 2);
+    painter->drawEllipse(QRectF(-smallRadius, -smallRadius, smallRadius * 2, smallRadius * 2));
 }
 
-QPainterPath MapRenderer::pointShape(const MapObject *object) const
+QPainterPath MapRenderer::pointInteractionShape(const MapObject *object) const
 {
     Q_ASSERT(object->shape() == MapObject::Point);
     QPainterPath path;
@@ -116,10 +147,14 @@ QPainterPath MapRenderer::pointShape(const MapObject *object) const
 
 void MapRenderer::setFlag(RenderFlag flag, bool enabled)
 {
+#if QT_VERSION >= 0x050700
+    mFlags.setFlag(flag, enabled);
+#else
     if (enabled)
         mFlags |= flag;
     else
         mFlags &= ~flag;
+#endif
 }
 
 /**
@@ -145,11 +180,7 @@ QPolygonF MapRenderer::lineToPolygon(const QPointF &start, const QPointF &end)
 
 QPen MapRenderer::makeGridPen(const QPaintDevice *device, QColor color) const
 {
-#if QT_VERSION >= 0x050600
     const qreal devicePixelRatio = device->devicePixelRatioF();
-#else
-    const int devicePixelRatio = device->devicePixelRatio();
-#endif
 
 #ifdef Q_OS_MAC
     const qreal dpiScale = 1.0;
@@ -192,8 +223,9 @@ static bool hasOpenGLEngine(const QPainter *painter)
             type == QPaintEngine::OpenGL2);
 }
 
-CellRenderer::CellRenderer(QPainter *painter, const CellType cellType)
+CellRenderer::CellRenderer(QPainter *painter, const MapRenderer *renderer, CellType cellType)
     : mPainter(painter)
+    , mRenderer(renderer)
     , mTile(nullptr)
     , mIsOpenGL(hasOpenGLEngine(painter))
     , mCellType(cellType)
@@ -305,6 +337,17 @@ void CellRenderer::render(const Cell &cell, const QPointF &pos, const QSizeF &si
     mPainter->setTransform(transform);
     mPainter->drawPixmap(target, image, source);
     mPainter->setTransform(oldTransform);
+
+    // A bit of a hack to still draw tile collision shapes when requested
+    if (mRenderer->flags().testFlag(ShowTileCollisionShapes)
+            && tile->objectGroup()
+            && !tile->objectGroup()->objects().isEmpty()) {
+        mTile = tile;
+        mFragments.append(fragment);
+        paintTileCollisionShapes();
+        mTile = nullptr;
+        mFragments.resize(0);
+    }
 }
 
 /**
@@ -319,6 +362,90 @@ void CellRenderer::flush()
                                   mFragments.size(),
                                   mTile->image());
 
+    if (mRenderer->flags().testFlag(ShowTileCollisionShapes)
+            && mTile->objectGroup()
+            && !mTile->objectGroup()->objects().isEmpty()) {
+        paintTileCollisionShapes();
+    }
+
     mTile = nullptr;
     mFragments.resize(0);
 }
+
+/**
+ * Returns a transform that rotates by \a rotation degrees around the given
+ * \a position.
+ */
+static QTransform rotateAt(const QPointF &position, qreal rotation)
+{
+    QTransform transform;
+    transform.translate(position.x(), position.y());
+    transform.rotate(rotation);
+    transform.translate(-position.x(), -position.y());
+    return transform;
+}
+
+void CellRenderer::paintTileCollisionShapes()
+{
+    const Tileset *tileset = mTile->tileset();
+    const Map map(tileset->orientation() == Tileset::Orthogonal ? Map::Orthogonal
+                                                                : Map::Isometric,
+                  QSize(1, 1),
+                  tileset->gridSize());
+
+    std::unique_ptr<MapRenderer> renderer;
+
+    const bool isIsometric = tileset->orientation() == Tileset::Isometric;
+    if (isIsometric)
+        renderer = std::make_unique<IsometricRenderer>(&map);
+    else
+        renderer = std::make_unique<OrthogonalRenderer>(&map);
+
+    const qreal lineWidth = mRenderer->objectLineWidth();
+    const qreal shadowDist = (lineWidth == 0 ? 1 : lineWidth) / mRenderer->painterScale();
+    const QPointF shadowOffset = QPointF(shadowDist * 0.5, shadowDist * 0.5);
+
+    QPen shadowPen(Qt::black);
+    shadowPen.setCosmetic(true);
+    shadowPen.setJoinStyle(Qt::RoundJoin);
+    shadowPen.setCapStyle(Qt::RoundCap);
+    shadowPen.setWidthF(lineWidth);
+    shadowPen.setStyle(Qt::DotLine);
+
+    mPainter->setRenderHint(QPainter::Antialiasing);
+
+    for (const auto &fragment : qAsConst(mFragments)) {
+        QTransform tileTransform;
+        tileTransform.translate(fragment.x, fragment.y);
+        tileTransform.rotate(fragment.rotation);
+        tileTransform.scale(fragment.scaleX, fragment.scaleY);
+        tileTransform.translate(-fragment.width * 0.5, -fragment.height * 0.5);
+
+        if (isIsometric)
+            tileTransform.translate(0, fragment.height - tileset->gridSize().height());
+
+        for (MapObject *object : mTile->objectGroup()->objects()) {
+            QColor penColor = object->effectiveColor();
+            QColor brushColor = penColor;
+            brushColor.setAlpha(50);
+            QPen colorPen(shadowPen);
+            colorPen.setColor(penColor);
+
+            mPainter->setPen(colorPen);
+            mPainter->setBrush(brushColor);
+
+            auto transform = rotateAt(renderer->pixelToScreenCoords(object->position()),
+                                      object->rotation());
+            transform *= tileTransform;
+
+            const auto shape = transform.map(renderer->shape(object));
+
+            mPainter->strokePath(shape.translated(shadowOffset), shadowPen);
+
+            if (object->shape() == MapObject::Polyline)
+                mPainter->strokePath(shape, colorPen);
+            else
+                mPainter->drawPath(shape);
+        }
+    }
+}
