@@ -28,10 +28,7 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef OBJECTGROUP_H
-#define OBJECTGROUP_H
-
-#include "tiled_global.h"
+#pragma once
 
 #include "layer.h"
 
@@ -39,6 +36,8 @@
 #include <QList>
 #include <QMetaType>
 
+#include <memory>
+
 namespace Tiled {
 
 class MapObject;
@@ -48,6 +47,8 @@ class MapObject;
  */
 class TILEDSHARED_EXPORT ObjectGroup : public Layer
 {
+    Q_OBJECT
+
 public:
     /**
      * Objects within an object group can either be drawn top down (sorted
@@ -61,20 +62,10 @@ public:
         IndexOrder
     };
 
-    /**
-     * Default constructor.
-     */
-    ObjectGroup();
+    explicit ObjectGroup(const QString &name = QString());
+    ObjectGroup(const QString &name, int x, int y);
 
-    /**
-     * Constructor with some parameters.
-     */
-    ObjectGroup(const QString &name, int x, int y, int width, int height);
-
-    /**
-     * Destructor.
-     */
-    ~ObjectGroup();
+    ~ObjectGroup() override;
 
     /**
      * Returns a pointer to the list of objects in this object group.
@@ -95,6 +86,7 @@ public:
      * Adds an object to this object group.
      */
     void addObject(MapObject *object);
+    void addObject(std::unique_ptr<MapObject> &&object);
 
     /**
      * Inserts an object at the specified index. This is only used for undoing
@@ -168,8 +160,8 @@ public:
     void offsetObjects(const QPointF &offset, const QRectF &bounds,
                        bool wrapX, bool wrapY);
 
-    bool canMergeWith(Layer *other) const override;
-    Layer *mergedWith(Layer *other) const override;
+    bool canMergeWith(const Layer *other) const override;
+    Layer *mergedWith(const Layer *other) const override;
 
     const QColor &color() const;
     void setColor(const QColor &color);
@@ -177,7 +169,16 @@ public:
     DrawOrder drawOrder() const;
     void setDrawOrder(DrawOrder drawOrder);
 
-    Layer *clone() const override;
+    ObjectGroup *clone() const override;
+
+    void resetObjectIds();
+    int highestObjectId() const;
+
+    // Enable easy iteration over objects with range-based for
+    QList<MapObject*>::iterator begin() { return mObjects.begin(); }
+    QList<MapObject*>::iterator end() { return mObjects.end(); }
+    QList<MapObject*>::const_iterator begin() const { return mObjects.begin(); }
+    QList<MapObject*>::const_iterator end() const { return mObjects.end(); }
 
 protected:
     ObjectGroup *initializeClone(ObjectGroup *clone) const;
@@ -239,5 +240,3 @@ TILEDSHARED_EXPORT ObjectGroup::DrawOrder drawOrderFromString(const QString &);
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::ObjectGroup*)
-
-#endif // OBJECTGROUP_H
