@@ -29,6 +29,7 @@
 
 #include "tileset.h"
 
+#include "imagecache.h"
 #include "terrain.h"
 #include "tile.h"
 #include "tilesetformat.h"
@@ -37,7 +38,9 @@
 
 #include <QBitmap>
 
-using namespace Tiled;
+#include "qtcompat_p.h"
+
+namespace Tiled {
 
 SharedTileset Tileset::create(const QString &name, int tileWidth, int tileHeight, int tileSpacing, int margin)
 {
@@ -233,7 +236,7 @@ bool Tileset::loadFromImage(const QImage &image, const QUrl &source)
     }
 
     // Blank out any remaining tiles to avoid confusion (todo: could be more clear)
-    for (Tile *tile : mTiles) {
+    for (Tile *tile : qAsConst(mTiles)) {
         if (tile->id() >= tileNum) {
             QPixmap tilePixmap = QPixmap(tileSize);
             tilePixmap.fill();
@@ -254,6 +257,8 @@ bool Tileset::loadFromImage(const QImage &image, const QUrl &source)
  * Exists only because the Python plugin interface does not handle QUrl (would
  * be nice to add this). Assumes \a source is a local file when it would
  * otherwise be a relative URL (without scheme).
+ *
+ * \sa setImageSource
  */
 bool Tileset::loadFromImage(const QImage &image, const QString &source)
 {
@@ -261,6 +266,19 @@ bool Tileset::loadFromImage(const QImage &image, const QString &source)
     return loadFromImage(image, url.isRelative() ? QUrl::fromLocalFile(source) : url);
 }
 
+/**
+ * Convenience override that loads the image via the ImageCache.
+ */
+bool Tileset::loadFromImage(const QString &fileName)
+{
+    const QUrl oldImageSource = mImageReference.source;
+    mImageReference.source = QUrl::fromLocalFile(fileName);
+    if (mImageReference.source != oldImageSource)
+        TilesetManager::instance()->tilesetImageSourceChanged(*this, oldImageSource);
+
+    return loadImage();
+}
+
 /**
  * Tries to load the image this tileset is referring to.
  *
@@ -269,7 +287,55 @@ bool Tileset::loadFromImage(const QImage &image, const QString &source)
  */
 bool Tileset::loadImage()
 {
-    return loadFromImage(mImageReference.create(), mImageReference.source);
+    TilesheetParameters p;
+    p.fileName = Tiled::urlToLocalFileOrQrc(mImageReference.source);
+    p.tileWidth = mTileWidth;
+    p.tileHeight = mTileHeight;
+    p.spacing = mTileSpacing;
+    p.margin = mMargin;
+    p.transparentColor = mImageReference.transparentColor;
+
+    if (p.tileWidth <= 0 || p.tileHeight <= 0) {
+        mImageReference.status = LoadingError;
+        return false;
+    }
+
+    QImage image = ImageCache::loadImage(p.fileName);
+    if (image.isNull()) {
+        mImageReference.status = LoadingError;
+        return false;
+    }
+
+    auto tiles = ImageCache::cutTiles(p);
+
+    for (int tileNum = 0; tileNum < tiles.size(); ++tileNum) {
+        auto it = mTiles.find(tileNum);
+        if (it != mTiles.end())
+            it.value()->setImage(tiles.at(tileNum));
+        else
+            mTiles.insert(tileNum, new Tile(tiles.at(tileNum), tileNum, this));
+    }
+
+    QPixmap blank;
+
+    // Blank out any remaining tiles to avoid confusion (todo: could be more clear)
+    for (Tile *tile : qAsConst(mTiles)) {
+        if (tile->id() >= tiles.size()) {
+            if (blank.isNull()) {
+                blank = QPixmap(mTileWidth, mTileHeight);
+                blank.fill();
+            }
+            tile->setImage(blank);
+        }
+    }
+
+    mNextTileId = std::max(mNextTileId, tiles.size());
+
+    mImageReference.size = image.size();
+    mColumnCount = columnCountForWidth(mImageReference.size.width());
+    mImageReference.status = LoadingReady;
+
+    return true;
 }
 
 /**
@@ -337,6 +403,19 @@ void Tileset::setImageSource(const QUrl &imageSource)
     }
 }
 
+/**
+ * Exists only because the Python plugin interface does not handle QUrl (would
+ * be nice to add this). Assumes \a source is a local file when it would
+ * otherwise be a relative URL (without scheme).
+ *
+ * \sa loadFromImage
+ */
+void Tileset::setImageSource(const QString &source)
+{
+    const QUrl url(source);
+    setImageSource(url.isRelative() ? QUrl::fromLocalFile(source) : url);
+}
+
 /**
  * Returns the column count that this tileset would have if the tileset
  * image would have the given \a width. This takes into account the tile
@@ -392,7 +471,7 @@ void Tileset::insertTerrain(int index, Terrain *terrain)
         mTerrainTypes.at(terrainId)->mId = terrainId;
 
     // Adjust tile terrain references
-    for (Tile *tile : mTiles) {
+    for (Tile *tile : qAsConst(mTiles)) {
         for (int corner = 0; corner < 4; ++corner) {
             const int terrainId = tile->cornerTerrainId(corner);
             if (terrainId >= index)
@@ -420,7 +499,7 @@ Terrain *Tileset::takeTerrainAt(int index)
         mTerrainTypes.at(terrainId)->mId = terrainId;
 
     // Clear and adjust tile terrain references
-    for (Tile *tile : mTiles) {
+    for (Tile *tile : qAsConst(mTiles)) {
         for (int corner = 0; corner < 4; ++corner) {
             const int terrainId = tile->cornerTerrainId(corner);
             if (terrainId == index)
@@ -440,14 +519,18 @@ Terrain *Tileset::takeTerrainAt(int index)
  */
 void Tileset::swapTerrains(int index, int swapIndex)
 {
+#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
     mTerrainTypes.swap(index, swapIndex);
+#else
+    mTerrainTypes.swapItemsAt(index, swapIndex);
+#endif
 
     // Reassign terrain IDs
     mTerrainTypes.at(index)->mId = index;
     mTerrainTypes.at(swapIndex)->mId = swapIndex;
 
     // Clear and adjust tile terrain references
-    for (Tile *tile : mTiles) {
+    for (Tile *tile : qAsConst(mTiles)) {
         for (int corner = 0; corner < 4; ++corner) {
             const int terrainId = tile->cornerTerrainId(corner);
             if (terrainId == index)
@@ -506,7 +589,7 @@ void Tileset::recalculateTerrainDistances()
         QVector<int> distance(terrainCount() + 1, -1);
 
         // Check all tiles for transitions to other terrain types
-        for (const Tile *tile : mTiles) {
+        for (const Tile *tile : qAsConst(mTiles)) {
             if (!hasByteEqualTo(tile->terrain(), i))
                 continue;
 
@@ -586,6 +669,11 @@ void Tileset::addWangSet(WangSet *wangSet)
     mWangSets.append(wangSet);
 }
 
+void Tileset::addWangSet(std::unique_ptr<WangSet> &&wangSet)
+{
+    addWangSet(wangSet.release());
+}
+
 /**
  * @brief Tileset::insertWangSet Adds a wangSet.
  * @param wangSet A pointer to the wangset to add.
@@ -633,7 +721,7 @@ Tile *Tileset::addTile(const QPixmap &image, const QUrl &source)
 void Tileset::addTiles(const QList<Tile *> &tiles)
 {
     for (Tile *tile : tiles) {
-        Q_ASSERT(!mTiles.contains(tile->id()));
+        Q_ASSERT(tile->tileset() == this && !mTiles.contains(tile->id()));
         mTiles.insert(tile->id(), tile);
     }
 
@@ -648,7 +736,7 @@ void Tileset::addTiles(const QList<Tile *> &tiles)
 void Tileset::removeTiles(const QList<Tile *> &tiles)
 {
     for (Tile *tile : tiles) {
-        Q_ASSERT(mTiles.contains(tile->id()));
+        Q_ASSERT(tile->tileset() == this && mTiles.contains(tile->id()));
         mTiles.remove(tile->id());
     }
 
@@ -699,6 +787,24 @@ void Tileset::setTileImage(Tile *tile,
     }
 }
 
+void Tileset::setOriginalTileset(const SharedTileset &original)
+{
+    mOriginalTileset = original;
+}
+
+/**
+ * When a tileset gets exported, a copy might be made to apply certain export
+ * options. In this case, the copy will have a (weak) pointer to the original
+ * tileset, to allow issues found during export to refer to this tileset.
+ */
+SharedTileset Tileset::originalTileset() const
+{
+    SharedTileset original { mOriginalTileset };
+    if (!original)
+        original = sharedPointer();
+    return original;
+}
+
 void Tileset::swap(Tileset &other)
 {
     const Properties p = properties();
@@ -729,18 +835,18 @@ void Tileset::swap(Tileset &other)
     // Don't swap mWeakPointer, since it's a reference to this.
 
     // Update back references from tiles and terrains
-    for (auto tile : mTiles)
+    for (auto tile : qAsConst(mTiles))
         tile->mTileset = this;
-    for (auto terrain : mTerrainTypes)
+    for (auto terrain : qAsConst(mTerrainTypes))
         terrain->mTileset = this;
-    for (auto wangSet : mWangSets)
+    for (auto wangSet : qAsConst(mWangSets))
         wangSet->setTileset(this);
 
-    for (auto tile : other.mTiles)
+    for (auto tile : qAsConst(other.mTiles))
         tile->mTileset = &other;
-    for (auto terrain : other.mTerrainTypes)
+    for (auto terrain : qAsConst(other.mTerrainTypes))
         terrain->mTileset = &other;
-    for (auto wangSet : other.mWangSets)
+    for (auto wangSet : qAsConst(other.mWangSets))
         wangSet->setTileset(&other);
 }
 
@@ -750,13 +856,10 @@ SharedTileset Tileset::clone() const
     c->setProperties(properties());
 
     // mFileName stays empty
-    c->mImageReference = mImageReference;
     c->mTileOffset = mTileOffset;
     c->mOrientation = mOrientation;
     c->mGridSize = mGridSize;
     c->mColumnCount = mColumnCount;
-    c->mExpectedColumnCount = mExpectedColumnCount;
-    c->mExpectedRowCount = mExpectedRowCount;
     c->mNextTileId = mNextTileId;
     c->mTerrainDistancesDirty = mTerrainDistancesDirty;
     c->mStatus = mStatus;
@@ -781,6 +884,10 @@ SharedTileset Tileset::clone() const
     for (WangSet *wangSet : mWangSets)
         c->mWangSets.append(wangSet->clone(c.data()));
 
+    // Call setter to please TilesetManager, which starts watching the image of
+    // the tileset when it calls TilesetManager::tilesetImageSourceChanged.
+    c->setImageReference(mImageReference);
+
     return c;
 }
 
@@ -791,7 +898,7 @@ void Tileset::updateTileSize()
 {
     int maxWidth = 0;
     int maxHeight = 0;
-    for (Tile *tile : mTiles) {
+    for (Tile *tile : qAsConst(mTiles)) {
         const QSize size = tile->size();
         if (maxWidth < size.width())
             maxWidth = size.width();
@@ -821,3 +928,5 @@ Tileset::Orientation Tileset::orientationFromString(const QString &string)
         orientation = Isometric;
     return orientation;
 }
+
+} // namespace Tiled
