@@ -193,7 +193,7 @@ WangIdVariations::iterator &WangIdVariations::iterator::operator ++()
     while (true) {
         mCurrent.setIndexColor(currentSpot, mCurrent.indexColor(currentSpot) + 1);
 
-        if (mCurrent.indexColor(currentSpot) > (currentSpot & 1? mCornerColors : mEdgeColors)) {
+        if (mCurrent.indexColor(currentSpot) > ((currentSpot & 1) ? mCornerColors : mEdgeColors)) {
             mCurrent.setIndexColor(currentSpot, 0);
             if (++index >= mZeroSpots.size())
                 break;
@@ -394,7 +394,7 @@ void WangSet::setCornerColorCount(int n)
     }
 }
 
-void WangSet::insertWangColor(QSharedPointer<WangColor> wangColor)
+void WangSet::insertWangColor(const QSharedPointer<WangColor> &wangColor)
 {
     if (wangColor->isEdge())
         insertEdgeWangColor(wangColor);
@@ -403,7 +403,7 @@ void WangSet::insertWangColor(QSharedPointer<WangColor> wangColor)
 }
 
 
-void WangSet::addWangColor(QSharedPointer<WangColor> wangColor)
+void WangSet::addWangColor(const QSharedPointer<WangColor> &wangColor)
 {
     if (wangColor->isEdge()) {
         wangColor->setColorIndex(mEdgeColors.size() + 1);
@@ -414,7 +414,7 @@ void WangSet::addWangColor(QSharedPointer<WangColor> wangColor)
     }
 }
 
-void WangSet::insertEdgeWangColor(QSharedPointer<WangColor> wangColor)
+void WangSet::insertEdgeWangColor(const QSharedPointer<WangColor> &wangColor)
 {
     Q_ASSERT(edgeColorCount() + 1 >= wangColor->colorIndex());
 
@@ -425,7 +425,7 @@ void WangSet::insertEdgeWangColor(QSharedPointer<WangColor> wangColor)
         mEdgeColors.at(i)->setColorIndex(i + 1);
 }
 
-void WangSet::insertCornerWangColor(QSharedPointer<WangColor> wangColor)
+void WangSet::insertCornerWangColor(const QSharedPointer<WangColor> &wangColor)
 {
     Q_ASSERT(cornerColorCount() + 1 >= wangColor->colorIndex());
 
@@ -466,14 +466,14 @@ void WangSet::removeCornerWangColor(int color)
         mCornerColors.at(i)->setColorIndex(i + 1);
 }
 
-QSharedPointer<WangColor> WangSet::edgeColorAt(int index) const
+const QSharedPointer<WangColor> &WangSet::edgeColorAt(int index) const
 {
     Q_ASSERT(index > 0 && index <= edgeColorCount());
 
     return mEdgeColors.at(index - 1);
 }
 
-QSharedPointer<WangColor> WangSet::cornerColorAt(int index) const
+const QSharedPointer<WangColor> &WangSet::cornerColorAt(int index) const
 {
     Q_ASSERT(index > 0 && index <= cornerColorCount());
 
@@ -487,7 +487,7 @@ QList<Tile *> WangSet::tilesChangedOnSetEdgeColors(int newEdgeColors) const
     for (auto i = mTileInfoToWangId.cbegin(); i != mTileInfoToWangId.cend(); ++i) {
         if (!wangIdIsValid(i.value(), newEdgeColors, cornerColorCount())) {
             int tileId = i.key() & 0x1fffffff;
-            tiles.append(mTileset->tileAt(tileId));
+            tiles.append(mTileset->findTile(tileId));
         }
     }
 
@@ -501,7 +501,7 @@ QList<Tile *> WangSet::tilesChangedOnSetCornerColors(int newCornerColors) const
     for (auto i = mTileInfoToWangId.cbegin(); i != mTileInfoToWangId.cend(); ++i) {
         if (!wangIdIsValid(i.value(), edgeColorCount(), newCornerColors)) {
             int tileId = i.key() & 0x1fffffff;
-            tiles.append(mTileset->tileAt(tileId));
+            tiles.append(mTileset->findTile(tileId));
         }
     }
 
@@ -517,17 +517,17 @@ QList<Tile *> WangSet::tilesChangedOnRemoveColor(int color, bool isEdge) const
             int c = isEdge? i.value().edgeColor(j) : i.value().cornerColor(j);
             int tileId = i.key() & 0x1fffffff;
             if (c >= color) {
-                tiles.append(mTileset->tileAt(tileId));
+                tiles.append(mTileset->findTile(tileId));
                 break;
             }
             if (isEdge) {
                 if (edgeColorCount() == 2 && c) {
-                    tiles.append(mTileset->tileAt(tileId));
+                    tiles.append(mTileset->findTile(tileId));
                     break;
                 }
             } else {
                 if (cornerColorCount() == 2 && c) {
-                    tiles.append(mTileset->tileAt(tileId));
+                    tiles.append(mTileset->findTile(tileId));
                     break;
                 }
             }
@@ -593,7 +593,7 @@ void WangSet::removeWangTile(const WangTile &wangTile)
 QList<WangTile> WangSet::sortedWangTiles() const
 {
     QList<WangTile> wangTiles = mWangIdToWangTile.values();
-    qStableSort(wangTiles.begin(), wangTiles.end());
+    std::stable_sort(wangTiles.begin(), wangTiles.end());
     return wangTiles;
 }
 
@@ -784,18 +784,37 @@ WangId WangSet::templateWangIdAt(unsigned n) const
 
 WangSet *WangSet::clone(Tileset *tileset) const
 {
-    WangSet *c = new WangSet(*this);
-
     // Caller is responsible for adding the WangSet to this tileset
-    c->setTileset(tileset);
+    WangSet *c = new WangSet(tileset, mName, mImageTileId);
+
+    c->mUniqueFullWangIdCount = mUniqueFullWangIdCount;
+    c->mEdgeColors = mEdgeColors;
+    c->mCornerColors = mCornerColors;
+    c->mWangIdToWangTile = mWangIdToWangTile;
+    c->mTileInfoToWangId = mTileInfoToWangId;
+    c->setProperties(properties());
 
     // Avoid sharing Wang colors
     for (QSharedPointer<WangColor> &wangColor : c->mEdgeColors) {
-        wangColor = QSharedPointer<WangColor>::create(*wangColor);
+        const auto properties = wangColor->properties();
+        wangColor = QSharedPointer<WangColor>::create(wangColor->colorIndex(),
+                                                      wangColor->isEdge(),
+                                                      wangColor->name(),
+                                                      wangColor->color(),
+                                                      wangColor->imageId(),
+                                                      wangColor->probability());
+        wangColor->setProperties(properties);
         wangColor->mWangSet = c;
     }
     for (QSharedPointer<WangColor> &wangColor : c->mCornerColors) {
-        wangColor = QSharedPointer<WangColor>::create(*wangColor);
+        const auto properties = wangColor->properties();
+        wangColor = QSharedPointer<WangColor>::create(wangColor->colorIndex(),
+                                                      wangColor->isEdge(),
+                                                      wangColor->name(),
+                                                      wangColor->color(),
+                                                      wangColor->imageId(),
+                                                      wangColor->probability());
+        wangColor->setProperties(properties);
         wangColor->mWangSet = c;
     }
 
