@@ -27,6 +27,7 @@
 #include "tilelayer.h"
 #include "compression.h"
 
+#include <QCoreApplication>
 #include <QFile>
 
 namespace Droidcraft {
@@ -35,8 +36,7 @@ DroidcraftPlugin::DroidcraftPlugin()
 {
 }
 
-// Reader
-Tiled::Map *DroidcraftPlugin::read(const QString &fileName)
+std::unique_ptr<Tiled::Map> DroidcraftPlugin::read(const QString &fileName)
 {
     using namespace Tiled;
 
@@ -60,14 +60,14 @@ Tiled::Map *DroidcraftPlugin::read(const QString &fileName)
     // Build 48 x 48 map
     // Create a Map -> Create a Tileset -> Add Tileset to map
     // -> Create a TileLayer -> Fill layer -> Add TileLayer to Map
-    Map *map = new Map(Map::Orthogonal, 48, 48, 32, 32);
+    std::unique_ptr<Map> map { new Map(Map::Orthogonal, 48, 48, 32, 32) };
 
     SharedTileset mapTileset(Tileset::create("tileset", 32, 32));
     mapTileset->loadFromImage(QImage(":/tileset.png"), QUrl("qrc://tileset.png"));
     map->addTileset(mapTileset);
 
     // Fill layer
-    TileLayer *mapLayer = new TileLayer("map", 0, 0, 48, 48);
+    auto mapLayer = std::make_unique<TileLayer>("map", 0, 0, 48, 48);
 
     // Load
     for (int i = 0; i < 48 * 48; i++) {
@@ -80,7 +80,7 @@ Tiled::Map *DroidcraftPlugin::read(const QString &fileName)
         mapLayer->setCell(x, y, Cell(tile));
     }
 
-    map->addLayer(mapLayer);
+    map->addLayer(std::move(mapLayer));
 
     return map;
 }
@@ -90,9 +90,10 @@ bool DroidcraftPlugin::supportsFile(const QString &fileName) const
     return fileName.endsWith(QLatin1String(".dat"), Qt::CaseInsensitive);
 }
 
-// Writer
-bool DroidcraftPlugin::write(const Tiled::Map *map, const QString &fileName)
+bool DroidcraftPlugin::write(const Tiled::Map *map, const QString &fileName, Options options)
 {
+    Q_UNUSED(options)
+
     using namespace Tiled;
 
     // Check layer count and type
@@ -127,7 +128,7 @@ bool DroidcraftPlugin::write(const Tiled::Map *map, const QString &fileName)
     // Write QByteArray
     SaveFile file(fileName);
     if (!file.open(QIODevice::WriteOnly)) {
-        mError = tr("Could not open file for writing.");
+        mError = QCoreApplication::translate("File Errors", "Could not open file for writing.");
         return false;
     }
 
