@@ -32,12 +32,15 @@
 #include "tileset.h"
 #include "objectgroup.h"
 
-#include <QFileInfo>
+#include <QCoreApplication>
 #include <QDir>
+#include <QFileInfo>
 #include <QSettings>
 #include <QStringList>
 #include <QTextStream>
 
+#include <memory>
+
 using namespace Tiled;
 
 namespace Flare {
@@ -46,17 +49,17 @@ FlarePlugin::FlarePlugin()
 {
 }
 
-Tiled::Map *FlarePlugin::read(const QString &fileName)
+std::unique_ptr<Tiled::Map> FlarePlugin::read(const QString &fileName)
 {
     QFile file(fileName);
 
     if (!file.open (QIODevice::ReadOnly)) {
-        mError = tr("Could not open file for reading.");
+        mError = QCoreApplication::translate("File Errors", "Could not open file for reading.");
         return nullptr;
     }
 
-    // default to values of the original flare alpha game.
-    QScopedPointer<Map> map(new Map(Map::Isometric, 256, 256, 64, 32));
+    // default to values of the original Flare alpha game.
+    auto map = std::make_unique<Map>(Map::Isometric, 256, 256, 64, 32);
 
     QTextStream stream (&file);
     QString line;
@@ -267,7 +270,7 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
         return nullptr;
     }
 
-    return map.take();
+    return map;
 }
 
 bool FlarePlugin::supportsFile(const QString &fileName) const
@@ -290,12 +293,14 @@ QString FlarePlugin::errorString() const
     return mError;
 }
 
-bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName)
+bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName, Options options)
 {
+    Q_UNUSED(options)
+
     SaveFile file(fileName);
 
     if (!file.open(QFile::WriteOnly | QFile::Text)) {
-        mError = tr("Could not open file for writing.");
+        mError = QCoreApplication::translate("File Errors", "Could not open file for writing.");
         return false;
     }
 
