@@ -20,6 +20,7 @@
 
 #include "abstractobjecttool.h"
 
+#include "actionmanager.h"
 #include "addremovetileset.h"
 #include "changemapobject.h"
 #include "documentmanager.h"
@@ -98,15 +99,24 @@ AbstractObjectTool::AbstractObjectTool(Id id,
 
     mFlipHorizontal = new QAction(this);
     mFlipHorizontal->setIcon(flipHorizontalIcon);
+    mFlipHorizontal->setShortcut(Qt::Key_X);
 
     mFlipVertical = new QAction(this);
     mFlipVertical->setIcon(flipVerticalIcon);
+    mFlipVertical->setShortcut(Qt::Key_Y);
 
     mRotateLeft = new QAction(this);
     mRotateLeft->setIcon(rotateLeftIcon);
+    mRotateLeft->setShortcut(Qt::SHIFT + Qt::Key_Z);
 
     mRotateRight = new QAction(this);
     mRotateRight->setIcon(rotateRightIcon);
+    mRotateRight->setShortcut(Qt::Key_Z);
+
+    ActionManager::registerAction(mFlipHorizontal, "FlipHorizontal");
+    ActionManager::registerAction(mFlipVertical, "FlipVertical");
+    ActionManager::registerAction(mRotateLeft, "RotateLeft");
+    ActionManager::registerAction(mRotateRight, "RotateRight");
 
     connect(mFlipHorizontal, &QAction::triggered, this, &AbstractObjectTool::flipHorizontally);
     connect(mFlipVertical, &QAction::triggered, this, &AbstractObjectTool::flipVertically);
@@ -175,15 +185,10 @@ void AbstractObjectTool::mousePressed(QGraphicsSceneMouseEvent *event)
 
 void AbstractObjectTool::languageChanged()
 {
-    mFlipHorizontal->setToolTip(tr("Flip Horizontally"));
-    mFlipVertical->setToolTip(tr("Flip Vertically"));
-    mRotateLeft->setToolTip(QCoreApplication::translate("Tiled::StampActions", "Rotate Left"));
-    mRotateRight->setToolTip(QCoreApplication::translate("Tiled::StampActions", "Rotate Right"));
-
-    mFlipHorizontal->setShortcut(Qt::Key_X);
-    mFlipVertical->setShortcut(Qt::Key_Y);
-    mRotateLeft->setShortcut(Qt::SHIFT + Qt::Key_Z);
-    mRotateRight->setShortcut(Qt::Key_Z);
+    mFlipHorizontal->setText(tr("Flip Horizontally"));
+    mFlipVertical->setText(tr("Flip Vertically"));
+    mRotateLeft->setText(QCoreApplication::translate("Tiled::StampActions", "Rotate Left"));
+    mRotateRight->setText(QCoreApplication::translate("Tiled::StampActions", "Rotate Right"));
 }
 
 void AbstractObjectTool::populateToolBar(QToolBar *toolBar)
