@@ -28,12 +28,16 @@
 
 namespace Tiled {
 
-AbstractTool::AbstractTool(const QString &name, const QIcon &icon,
-                           const QKeySequence &shortcut, QObject *parent)
+AbstractTool::AbstractTool(Id id,
+                           const QString &name,
+                           const QIcon &icon,
+                           const QKeySequence &shortcut,
+                           QObject *parent)
     : QObject(parent)
     , mName(name)
     , mIcon(icon)
     , mShortcut(shortcut)
+    , mId(id)
     , mEnabled(false)
     , mToolManager(nullptr)
     , mMapDocument(nullptr)
