@@ -22,6 +22,7 @@
 #pragma once
 
 #include "changeevents.h"
+#include "id.h"
 
 #include <QCursor>
 #include <QGraphicsSceneMouseEvent>
@@ -58,6 +59,7 @@ class AbstractTool : public QObject
 {
     Q_OBJECT
 
+    Q_PROPERTY(QByteArray id READ idName CONSTANT)
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY changed)
     Q_PROPERTY(QIcon icon READ icon WRITE setIcon NOTIFY changed)
     Q_PROPERTY(QKeySequence shortcut READ shortcut WRITE setShortcut NOTIFY changed)
@@ -69,11 +71,15 @@ public:
     /**
      * Constructs an abstract tool with the given \a name and \a icon.
      */
-    AbstractTool(const QString &name,
+    AbstractTool(Id id,
+                 const QString &name,
                  const QIcon &icon,
                  const QKeySequence &shortcut,
                  QObject *parent = nullptr);
 
+    Id id() const;
+    QByteArray idName() const;
+
     QString name() const;
     void setName(const QString &name);
 
@@ -201,6 +207,7 @@ private:
     QKeySequence mShortcut;
     QString mStatusInfo;
     QCursor mCursor;
+    Id mId;
     bool mEnabled;
 
     ToolManager *mToolManager;
@@ -208,6 +215,16 @@ private:
 };
 
 
+inline Id AbstractTool::id() const
+{
+    return mId;
+}
+
+inline QByteArray AbstractTool::idName() const
+{
+    return mId.name();
+}
+
 inline QString AbstractTool::name() const
 {
     return mName;
