@@ -30,7 +30,6 @@
 #include <QSettings>
 
 using namespace Tiled;
-using namespace Tiled::Internal;
 
 static const char * const TYPE_KEY = "AddPropertyDialog/PropertyType";
 
@@ -38,7 +37,9 @@ AddPropertyDialog::AddPropertyDialog(QWidget *parent)
     : QDialog(parent)
     , mUi(new Ui::AddPropertyDialog)
 {
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
+#endif
 
     mUi->setupUi(this);
     resize(Utils::dpiScaled(size()));
