@@ -27,16 +27,16 @@
 #include <QEvent>
 #include <QMenu>
 #include <QMessageBox>
+#include <QPushButton>
 
 using namespace Tiled;
 using namespace Tiled::Utils;
-using namespace Tiled::Internal;
 
 CommandButton::CommandButton(QWidget *parent)
     : QToolButton(parent)
     , mMenu(new QMenu(this))
 {
-    setIcon(QIcon(QLatin1String(":images/24x24/system-run.png")));
+    setIcon(QIcon(QLatin1String(":images/24/system-run.png")));
     setThemeIcon(this, "system-run");
     retranslateUi();
 
@@ -54,24 +54,23 @@ void CommandButton::runCommand()
 
     QAction *action = dynamic_cast<QAction*>(sender());
     if (action && action->data().isValid()) {
-        //run the command passed by the action
+        // run the command passed by the action
         command = Command::fromQVariant(action->data());
     } else {
-        //run the default command
-        command = CommandDataModel().firstEnabledCommand();
-
-        if (!command.isEnabled) {
+        // run the default command
+        if (auto c = CommandManager::instance()->commandDataModel()->firstEnabledCommand()) {
+            command = *c;
+        } else {
             QMessageBox msgBox(window());
             msgBox.setIcon(QMessageBox::Warning);
             msgBox.setWindowTitle(tr("Error Executing Command"));
             msgBox.setText(tr("You do not have any commands setup."));
             msgBox.addButton(QMessageBox::Ok);
-            msgBox.addButton(tr("Edit commands..."), QMessageBox::ActionRole);
+            const auto editButton = msgBox.addButton(tr("Edit commands..."), QMessageBox::ActionRole);
             msgBox.setDefaultButton(QMessageBox::Ok);
             msgBox.setEscapeButton(QMessageBox::Ok);
 
-            QAbstractButton *button = msgBox.buttons().last();
-            connect(button, &QAbstractButton::clicked, this, &CommandButton::showDialog);
+            connect(editButton, &QAbstractButton::clicked, this, &CommandButton::showDialog);
 
             msgBox.exec();
             return;
