@@ -23,12 +23,10 @@
 #include "command.h"
 
 #include <QAbstractTableModel>
-#include <QSettings>
 
 class QMenu;
 
 namespace Tiled {
-namespace Internal {
 
 class CommandDataModel : public QAbstractTableModel
 {
@@ -49,7 +47,7 @@ public:
       */
     void commit();
 
-    Command firstEnabledCommand() const;
+    const Command *firstEnabledCommand() const;
 
     /**
       * Returns a list of all the commands.
@@ -127,8 +125,11 @@ public:
     bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row,
                       int column, const QModelIndex &parent) override;
 
-    void setCommand(const QModelIndex &index, const QString &value);
+    void setExecutable(const QModelIndex &index, const QString &value);
+    void setArguments(const QModelIndex &index, const QString &value);
+    void setWorkingDirectory(const QModelIndex &index, const QString &value);
     void setShortcut(const QModelIndex &index, const QKeySequence &value);
+    void setShowOutput(const QModelIndex &index, bool value);
     void setSaveBeforeExecute(const QModelIndex &index, bool value);
 
     Command command(const QModelIndex &index) const;
@@ -167,10 +168,7 @@ public slots:
     void remove(int commandIndex);
 
 private:
-
-    QSettings mSettings;
     QList<Command> mCommands;
 };
 
-} // namespace Internal
 } // namespace Tiled
