@@ -28,7 +28,6 @@ class CommandDialog;
 }
 
 namespace Tiled {
-namespace Internal {
 
 class CommandDataModel;
 
@@ -51,11 +50,19 @@ public slots:
 
     void setSaveBeforeExecute(int state);
 
-    void setCommand(const QString &text);
+    void setShowOutput(int state);
+
+    void setExecutable(const QString &text);
+
+    void setArguments(const QString &text);
+
+    void setWorkingDirectory(const QString &text);
 
     void updateWidgets(const QModelIndex &current, const QModelIndex&);
 
-    void openFileDialog();
+    void browseExecutable();
+
+    void browseWorkingDirectory();
 
 private:
     Ui::CommandDialog *mUi;
@@ -73,12 +80,13 @@ public:
       */
     CommandDataModel *model() const { return mModel; }
 
-private slots:
+protected:
     /**
       * Displays a context menu for the item at <i>event</i>'s position.
       */
     void contextMenuEvent(QContextMenuEvent *event) override;
 
+private:
     /**
       * Fixes the selection after rows have been removed.
       */
@@ -89,9 +97,7 @@ private slots:
       */
     void removeSelectedCommands();
 
-private:
     CommandDataModel *mModel;
 };
 
-} // namespace Internal
 } // namespace Tiled
