@@ -23,6 +23,7 @@
 #include "commanddatamodel.h"
 #include "commanddialog.h"
 #include "logginginterface.h"
+#include "pluginmanager.h"
 #include "utils.h"
 
 #include <QApplication>
@@ -30,18 +31,22 @@
 #include <QLatin1String>
 #include <QMenu>
 
+#include "qtcompat_p.h"
+
 namespace Tiled {
-namespace Internal {
 
 CommandManager *CommandManager::mInstance;
 
 CommandManager::CommandManager()
     : mModel(new CommandDataModel(this))
-    , mLogger(new LoggingInterface())
 {
     updateActions();
 }
 
+CommandManager::~CommandManager()
+{
+}
+
 CommandManager *CommandManager::instance()
 {
     if (!mInstance)
@@ -76,7 +81,7 @@ void CommandManager::showDialog()
 
 void CommandManager::populateMenus()
 {
-    for (QMenu *menu : mMenus) {
+    for (QMenu *menu : qAsConst(mMenus)) {
         menu->clear();
         menu->addActions(mActions);
     }
@@ -98,7 +103,7 @@ void CommandManager::updateActions()
         QAction *mAction = new QAction(command.name, this);
         mAction->setShortcut(command.shortcut);
 
-        connect(mAction, &QAction::triggered, [this,i]() { mModel->execute(i); });
+        connect(mAction, &QAction::triggered, [this,i] { mModel->execute(i); });
 
         mActions.append(mAction);
     }
@@ -111,7 +116,7 @@ void CommandManager::updateActions()
 
     mEditCommands = new QAction(this);
     mEditCommands->setIcon(
-            QIcon(QLatin1String(":/images/24x24/system-run.png")));
+            QIcon(QLatin1String(":/images/24/system-run.png")));
     Utils::setThemeIcon(mEditCommands, "system-run");
 
     connect(mEditCommands, &QAction::triggered, this, &CommandManager::showDialog);
@@ -127,5 +132,4 @@ void CommandManager::retranslateUi()
     mEditCommands->setText(tr("Edit Commands..."));
 }
 
-} // namespace Internal
 } // namespace Tiled
