@@ -1,6 +1,7 @@
 /*
  * consoledock.h
  * Copyright 2013, Samuli Tuomola <samuli.tuomola@gmail.com>
+ * Copyright 2018-2019, Thorbjørn Lindeijer <bjorn@lindeijer.nl>
  *
  * This file is part of Tiled.
  *
@@ -18,39 +19,42 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CONSOLEDOCK_H
-#define CONSOLEDOCK_H
+#pragma once
 
 #include <QDockWidget>
-#include <QPlainTextEdit>
+
+class QLineEdit;
+class QPlainTextEdit;
 
 namespace Tiled {
 
-class LoggingInterface;
-
-namespace Internal {
-
 class ConsoleDock : public QDockWidget
 {
     Q_OBJECT
-    
+
 public:
     explicit ConsoleDock(QWidget *parent = nullptr);
     ~ConsoleDock();
 
-private slots:
-    void appendInfo(const QString &str);
-    void appendError(const QString &str);
-
-    void onObjectAdded(QObject *object);
+protected:
+    void changeEvent(QEvent *e) override;
 
 private:
-    void registerOutput(LoggingInterface *output);
+    void appendInfo(const QString &str);
+    void appendWarning(const QString &str);
+    void appendError(const QString &str);
+    void appendScript(const QString &str);
 
-    QPlainTextEdit *plainTextEdit;
+    void executeScript();
+
+    void moveHistory(int direction);
+
+    void retranslateUi();
+
+    QPlainTextEdit *mPlainTextEdit;
+    QLineEdit *mLineEdit;
+    QStringList mHistory;
+    int mHistoryPosition = 0;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // CONSOLEDOCK_H
