@@ -29,7 +29,7 @@
 #include <QLocale>
 #include <QTranslator>
 
-using namespace Tiled::Internal;
+using namespace Tiled;
 
 LanguageManager *LanguageManager::mInstance;
 
@@ -51,7 +51,7 @@ LanguageManager::LanguageManager()
     , mAppTranslator(nullptr)
 {
     mTranslationsDir = QCoreApplication::applicationDirPath();
-#if defined(Q_OS_WIN32) || defined(TILED_LINUX_ARCHIVE)
+#if defined(Q_OS_WIN32)
     mTranslationsDir += QLatin1String("/translations");
 #elif defined(Q_OS_MAC)
     mTranslationsDir += QLatin1String("/../Translations");
