@@ -29,8 +29,7 @@ class GroupLayer;
 class Layer;
 class Map;
 
-namespace Internal {
-
+class ChangeEvent;
 class MapDocument;
 
 /**
@@ -84,23 +83,21 @@ public:
     void insertLayer(GroupLayer *parentLayer, int index, Layer *layer);
     Layer *takeLayerAt(GroupLayer *parentLayer, int index);
     void replaceLayer(Layer *layer, Layer *replacement);
+    void moveLayer(GroupLayer *parentLayer, int index, GroupLayer *toParentLayer, int toIndex);
 
-    void setLayerVisible(Layer *layer, bool visible);
-    void setLayerLocked(Layer *layer, bool locked);
-    void setLayerOpacity(Layer *layer, qreal opacity);
-    void setLayerOffset(Layer *layer, const QPointF &offset);
-
-    void renameLayer(Layer *layer, const QString &name);
-
-    void toggleOtherLayers(Layer *layer);
+    void toggleLayers(const QList<Layer *> &layers);
+    void toggleLockLayers(const QList<Layer *> &layers);
+    void toggleOtherLayers(const QList<Layer *> &layers);
+    void toggleLockOtherLayers(const QList<Layer *> &layers);
 
 signals:
     void layerAdded(Layer *layer);
     void layerAboutToBeRemoved(GroupLayer *parentLayer, int index);
     void layerRemoved(Layer *layer);
-    void layerChanged(Layer *layer);
 
 private:
+    void documentChanged(const ChangeEvent &change);
+
     MapDocument *mMapDocument;
     Map *mMap;
 
@@ -117,5 +114,4 @@ inline MapDocument *LayerModel::mapDocument() const
     return mMapDocument;
 }
 
-} // namespace Internal
 } // namespace Tiled
