@@ -25,7 +25,6 @@
 class QToolButton;
 
 namespace Tiled {
-namespace Internal {
 
 class CommandButton;
 class Document;
@@ -40,12 +39,11 @@ public:
 protected:
     void changeEvent(QEvent *event) override;
 
-private slots:
+private:
     void onOrientationChanged(Qt::Orientation orientation);
 
     void currentDocumentChanged(Document *document);
 
-private:
     void retranslateUi();
 
     QToolButton *mNewButton;
@@ -56,5 +54,4 @@ private:
     CommandButton *mCommandButton;
 };
 
-} // namespace Internal
 } // namespace Tiled
