@@ -345,6 +345,9 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     undoAction->setShortcuts(QKeySequence::Undo);
     redoAction->setShortcuts(redoShortcuts);
 
+    ActionManager::registerAction(undoAction, "Undo");
+    ActionManager::registerAction(redoAction, "Redo");
+
     auto snappingGroup = new QActionGroup(this);
     mUi->actionSnapNothing->setActionGroup(snappingGroup);
     mUi->actionSnapToGrid->setActionGroup(snappingGroup);
@@ -678,6 +681,13 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
 #ifdef Q_OS_WIN
     connect(preferences, &Preferences::useOpenGLChanged, this, &MainWindow::ensureHasBorderInFullScreen);
 #endif
+#ifdef Q_OS_MAC
+    setUnifiedTitleAndToolBarOnMac(!preferences->useOpenGL());
+    connect(preferences, &Preferences::useOpenGLChanged, this, [this] (bool useOpenGL) {
+        if (useOpenGL)
+            setUnifiedTitleAndToolBarOnMac(false);
+    });
+#endif
 
     connect(preferences, &Preferences::recentFilesChanged, this, &MainWindow::updateRecentFilesMenu);
 
