@@ -29,6 +29,7 @@
 #include "tiled.h"
 #include "tileset.h"
 
+class QAction;
 class QComboBox;
 class QLabel;
 class QMainWindow;
@@ -38,34 +39,32 @@ class QToolButton;
 
 namespace Tiled {
 
-class ObjectTemplate;
 class Terrain;
 
-namespace Internal {
-
 class AbstractTool;
 class BucketFillTool;
+class ComboBoxProxyModel;
+class EditableMap;
 class EditPolygonTool;
 class LayerDock;
 class MapDocument;
-class MapDocumentActionHandler;
-class MapsDock;
 class MapView;
+class MapsDock;
 class MiniMapDock;
 class ObjectsDock;
-class TemplatesDock;
 class PropertiesDock;
 class ReversingProxyModel;
 class ShapeFillTool;
 class StampBrush;
+class TemplatesDock;
 class TerrainBrush;
 class TerrainDock;
-class TilesetDock;
 class TileStamp;
 class TileStampManager;
+class TileStampsDock;
+class TilesetDock;
 class ToolManager;
 class TreeViewComboBox;
-class ComboBoxProxyModel;
 class UndoDock;
 class WangBrush;
 class WangDock;
@@ -75,10 +74,16 @@ class MapEditor : public Editor
 {
     Q_OBJECT
 
+    Q_PROPERTY(Tiled::TilesetDock *tilesetsView READ tilesetDock)
+    Q_PROPERTY(Tiled::EditableMap *currentBrush READ currentBrush WRITE setCurrentBrush)
+    Q_PROPERTY(Tiled::MapView *currentMapView READ currentMapView)
+
 public:
     explicit MapEditor(QObject *parent = nullptr);
     ~MapEditor() override;
 
+    TilesetDock *tilesetDock() const { return mTilesetDock; }
+
     void saveState() override;
     void restoreState() override;
 
@@ -102,9 +107,22 @@ public:
     MapView *currentMapView() const;
     Zoomable *zoomable() const override;
 
+    void saveDocumentState(MapDocument *mapDocument);
+
     void showMessage(const QString &text, int timeout = 0);
 
-public slots:
+    void setCurrentTileset(const SharedTileset &tileset);
+    SharedTileset currentTileset() const;
+
+    EditableMap *currentBrush() const;
+    void setCurrentBrush(EditableMap *editableMap);
+
+    void addExternalTilesets(const QStringList &fileNames);
+
+    QAction *actionSelectNextTileset() const;
+    QAction *actionSelectPreviousTileset() const;
+
+private:
     void setSelectedTool(AbstractTool *tool);
 
     void paste(ClipboardManager::PasteFlags flags);
@@ -117,12 +135,8 @@ public slots:
 
     void selectWangBrush();
 
-    void addExternalTilesets(const QStringList &fileNames);
     void filesDroppedOnTilesetDock(const QStringList &fileNames);
 
-    void updateTemplateInstances(const ObjectTemplate *objectTemplate);
-
-private slots:
     void currentWidgetChanged();
 
     void cursorChanged(const QCursor &cursor);
@@ -132,9 +146,9 @@ private slots:
     void layerComboActivated();
     void updateLayerComboIndex();
 
-private:
     void setupQuickStamps();
     void retranslateUi();
+    void showTileCollisionShapesChanged(bool enabled);
 
     void handleExternalTilesetsAndImages(const QStringList &fileNames,
                                          bool handleImages);
@@ -157,7 +171,7 @@ private:
     TerrainDock *mTerrainDock;
     WangDock *mWangDock;
     MiniMapDock* mMiniMapDock;
-    QDockWidget *mTileStampsDock;
+    TileStampsDock *mTileStampsDock;
 
     TreeViewComboBox *mLayerComboBox;
     ComboBoxProxyModel *mComboBoxProxyModel;
@@ -197,5 +211,4 @@ inline MapView *MapEditor::currentMapView() const
     return viewForDocument(mCurrentMapDocument);
 }
 
-} // namespace Internal
 } // namespace Tiled
