@@ -24,11 +24,11 @@
 #pragma once
 
 #include "mapdocument.h"
+#include "mapitem.h"
 
 #include <QColor>
 #include <QGraphicsScene>
-#include <QMap>
-#include <QSet>
+#include <QHash>
 
 namespace Tiled {
 
@@ -39,14 +39,11 @@ class Tile;
 class TileLayer;
 class Tileset;
 
-namespace Internal {
-
 class AbstractTool;
 class LayerItem;
 class MapDocument;
 class MapObjectItem;
 class MapScene;
-class MapItem;
 class ObjectGroupItem;
 
 /**
@@ -63,14 +60,16 @@ public:
     MapDocument *mapDocument() const;
     void setMapDocument(MapDocument *map);
 
-    void enableSelectedTool();
-    void disableSelectedTool();
+    void setShowTileCollisionShapes(bool enabled);
+
+    QRectF mapBoundingRect() const;
 
     void setSelectedTool(AbstractTool *tool);
 
-protected:
-    void drawForeground(QPainter *painter, const QRectF &rect) override;
+signals:
+    void mapDocumentChanged(MapDocument *mapDocument);
 
+protected:
     bool event(QEvent *event) override;
 
     void keyPressEvent(QKeyEvent *event) override;
@@ -84,37 +83,28 @@ protected:
     void dragLeaveEvent(QGraphicsSceneDragDropEvent *event) override;
     void dragMoveEvent(QGraphicsSceneDragDropEvent *event) override;
 
-private slots:
-    void setGridVisible(bool visible);
-
+private:
     void refreshScene();
 
-    void currentLayerChanged();
-
     void mapChanged();
     void repaintTileset(Tileset *tileset);
-    void tileLayerChanged(TileLayer *, MapDocument::TileLayerChangeFlags flags);
 
-    void layerChanged(Layer *);
+    void tilesetReplaced(int index, Tileset *tileset, Tileset *oldTileset);
 
-    void adaptToTilesetTileSizeChanges();
-    void adaptToTileSizeChanges();
-
-    void tilesetReplaced();
-
-private:
     void updateDefaultBackgroundColor();
     void updateSceneRect();
 
+    MapItem *takeOrCreateMapItem(const MapDocumentPtr &mapDocument,
+                                 MapItem::DisplayMode displayMode);
+
     bool eventFilter(QObject *object, QEvent *event) override;
 
-    MapDocument *mMapDocument;
-    MapItem *mMapItem;
-    AbstractTool *mSelectedTool;
-    AbstractTool *mActiveTool;
-    bool mGridVisible;
-    bool mUnderMouse;
-    Qt::KeyboardModifiers mCurrentModifiers;
+    MapDocument *mMapDocument = nullptr;
+    QHash<MapDocument*, MapItem*> mMapItems;
+    AbstractTool *mSelectedTool = nullptr;
+    bool mUnderMouse = false;
+    bool mShowTileCollisionShapes = false;
+    Qt::KeyboardModifiers mCurrentModifiers = Qt::NoModifier;
     QPointF mLastMousePos;
     QColor mDefaultBackgroundColor;
 };
@@ -127,5 +117,4 @@ inline MapDocument *MapScene::mapDocument() const
     return mMapDocument;
 }
 
-} // namespace Internal
 } // namespace Tiled
