@@ -39,7 +39,6 @@
 #include <QPushButton>
 
 using namespace Tiled;
-using namespace Tiled::Internal;
 
 /**
  * Class represents the file system model with disabled dragging of directories.
@@ -168,21 +167,11 @@ MapsView::MapsView(QWidget *parent)
 
     mFileSystemModel = new FileSystemModel(this);
     mFileSystemModel->setRootPath(mapsDir.absolutePath());
-
-    QStringList nameFilters;
-
-    for (MapFormat *format : PluginManager::objects<MapFormat>()) {
-        if (!(format->capabilities() & MapFormat::Read))
-            continue;
-
-        const QString filter = format->nameFilter();
-        nameFilters.append(Utils::cleanFilterList(filter));
-    }
-
     mFileSystemModel->setFilter(QDir::AllDirs | QDir::Files | QDir::NoDot);
-    mFileSystemModel->setNameFilters(nameFilters);
     mFileSystemModel->setNameFilterDisables(false); // hide filtered files
 
+    updateNameFilters();
+
     setModel(mFileSystemModel);
 
     QHeaderView *headerView = header();
@@ -197,6 +186,11 @@ MapsView::MapsView(QWidget *parent)
 
     connect(this, &QAbstractItemView::activated,
             this, &MapsView::onActivated);
+
+    connect(PluginManager::instance(), &PluginManager::objectAdded,
+            this, &MapsView::pluginObjectAddedOrRemoved);
+    connect(PluginManager::instance(), &PluginManager::objectRemoved,
+            this, &MapsView::pluginObjectAddedOrRemoved);
 }
 
 QSize MapsView::sizeHint() const
@@ -237,3 +231,26 @@ void MapsView::onActivated(const QModelIndex &index)
 
     DocumentManager::instance()->openFile(path);
 }
+
+void MapsView::pluginObjectAddedOrRemoved(QObject *object)
+{
+    if (auto format = qobject_cast<MapFormat*>(object))
+        if (format->capabilities() & FileFormat::Read)
+            updateNameFilters();
+}
+
+void MapsView::updateNameFilters()
+{
+    QStringList nameFilters;
+
+    const auto mapFormats = PluginManager::objects<MapFormat>();
+    for (MapFormat *format : mapFormats) {
+        if (!(format->capabilities() & MapFormat::Read))
+            continue;
+
+        const QString filter = format->nameFilter();
+        nameFilters.append(Utils::cleanFilterList(filter));
+    }
+
+    mFileSystemModel->setNameFilters(nameFilters);
+}
