@@ -30,7 +30,6 @@ class QModelIndex;
 class QTreeView;
 
 namespace Tiled {
-namespace Internal {
 
 class MapsView;
 
@@ -41,15 +40,14 @@ class MapsDock : public QDockWidget
 public:
     MapsDock(QWidget *parent = nullptr);
 
-private slots:
-    void browse();
-    void editedMapsDirectory();
-    void onMapsDirectoryChanged();
-
 protected:
     void changeEvent(QEvent *e) override;
 
 private:
+    void browse();
+    void editedMapsDirectory();
+    void onMapsDirectoryChanged();
+
     void retranslateUi();
 
     QLineEdit *mDirectoryEdit;
@@ -75,13 +73,15 @@ public:
 
     QFileSystemModel *model() const { return mFileSystemModel; }
 
-private slots:
+private:
     void onMapsDirectoryChanged();
     void onActivated(const QModelIndex &index);
 
-private:
+    void pluginObjectAddedOrRemoved(QObject *object);
+
+    void updateNameFilters();
+
     QFileSystemModel *mFileSystemModel;
 };
 
-} // namespace Internal
 } // namespace Tiled
