@@ -24,8 +24,10 @@
 #include <QPinchGesture>
 
 namespace Tiled {
-namespace Internal {
 
+class MapObject;
+
+class MapDocument;
 class MapScene;
 class Zoomable;
 
@@ -40,6 +42,8 @@ class MapView : public QGraphicsView
 {
     Q_OBJECT
 
+    Q_PROPERTY(qreal scale READ scale WRITE setScale)
+
 public:
     /**
      * Using Qt::WA_StaticContents gives a performance boost in certain
@@ -55,21 +59,34 @@ public:
     };
 
     MapView(QWidget *parent = nullptr, Mode mode = StaticContents);
-    ~MapView();
+    ~MapView() override;
 
+    void setViewInitialized();
+
+    void setScene(MapScene *scene);
     MapScene *mapScene() const;
 
     Zoomable *zoomable() const { return mZoomable; }
 
+    qreal scale() const;
+    void setScale(qreal scale);
+
+    void fitMapInView();
+
     bool handScrolling() const { return mHandScrolling; }
     void setHandScrolling(bool handScrolling);
 
+    using QGraphicsView::centerOn;
+    Q_INVOKABLE void centerOn(qreal x, qreal y) { forceCenterOn(QPointF(x, y)); };
+
     void forceCenterOn(const QPointF &pos);
 
 protected:
     bool event(QEvent *event) override;
 
+    void showEvent(QShowEvent *) override;
     void hideEvent(QHideEvent *) override;
+    void resizeEvent(QResizeEvent *event) override;
 
     void keyPressEvent(QKeyEvent *event) override;
 
@@ -79,21 +96,39 @@ protected:
     void mouseReleaseEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
 
+    void focusInEvent(QFocusEvent *event) override;
+
     void handlePinchGesture(QPinchGesture *pinch);
 
-    void adjustCenterFromMousePosition(QPoint &mousePos);
+    void adjustCenterFromMousePosition(QPoint mousePos);
 
-private slots:
-    void adjustScale(qreal scale);
-    void setUseOpenGL(bool useOpenGL);
+signals:
+    void focused();
 
 private:
+    void adjustScale(qreal scale);
+    void setUseOpenGL(bool useOpenGL);
+    void updateSceneRect(const QRectF &sceneRect);
+    void updateSceneRect(const QRectF &sceneRect, const QTransform &transform);
+    void focusMapObject(MapObject *mapObject);
+
+    void setMapDocument(MapDocument *mapDocument);
+
+    MapDocument *mMapDocument = nullptr;
     QPoint mLastMousePos;
     QPointF mLastMouseScenePos;
-    bool mHandScrolling;
+    bool mHandScrolling = false;
+    bool mViewInitialized = false;
     Mode mMode;
     Zoomable *mZoomable;
 };
 
-} // namespace Internal
+
+inline void MapView::setViewInitialized()
+{
+    mViewInitialized = true;
+}
+
 } // namespace Tiled
+
+Q_DECLARE_METATYPE(Tiled::MapView*)
