@@ -23,8 +23,12 @@
 
 #include "actionmanager.h"
 #include "documentmanager.h"
+#include "issuescounter.h"
+#include "newsbutton.h"
+#include "newversionbutton.h"
 
 #include <QAction>
+#include <QStatusBar>
 
 namespace Tiled {
 
@@ -34,6 +38,20 @@ NoEditorWidget::NoEditorWidget(QWidget *parent) :
 {
     ui->setupUi(this);
 
+    // Transfer margin and spacing to the internal layout
+    ui->verticalLayout->setMargin(ui->gridLayout->margin());
+    ui->verticalLayout->setSpacing(ui->gridLayout->spacing());
+    ui->gridLayout->setMargin(0);
+    ui->gridLayout->setSpacing(0);
+
+    // Add a status bar to the bottom
+    auto statusBar = new QStatusBar;
+    statusBar->addPermanentWidget(new NewsButton(statusBar));
+    statusBar->addPermanentWidget(new NewVersionButton(NewVersionButton::AutoVisible, statusBar));
+    statusBar->addWidget(new IssuesCounter(statusBar));
+
+    ui->gridLayout->addWidget(statusBar, 3, 0, 1, 3);
+
     connect(ui->newMapButton, &QPushButton::clicked, this, &NoEditorWidget::newMap);
     connect(ui->newTilesetButton, &QPushButton::clicked, this, &NoEditorWidget::newTileset);
     connect(ui->openFileButton, &QPushButton::clicked, this, &NoEditorWidget::openFile);
