@@ -23,7 +23,6 @@
 #include <QWidget>
 
 namespace Tiled {
-namespace Internal {
 
 namespace Ui {
 class NoEditorWidget;
@@ -34,19 +33,18 @@ class NoEditorWidget : public QWidget
     Q_OBJECT
 
 public:
-    explicit NoEditorWidget(QWidget *parent = 0);
+    explicit NoEditorWidget(QWidget *parent = nullptr);
     ~NoEditorWidget();
 
 protected:
     void changeEvent(QEvent *e);
 
-private slots:
+private:
     void newMap();
     void newTileset();
+    void openFile();
 
-private:
     Ui::NoEditorWidget *ui;
 };
 
-} // namespace Internal
 } // namespace Tiled
