@@ -20,6 +20,7 @@
 
 #include "objectselectionitem.h"
 
+#include "geometry.h"
 #include "grouplayer.h"
 #include "map.h"
 #include "mapdocument.h"
@@ -39,95 +40,9 @@
 #include <cmath>
 
 namespace Tiled {
-namespace Internal {
-
-static const qreal labelMargin = 3;
-static const qreal labelDistance = 12;
-
-// TODO: Unduplicate the following helper functions between this and
-// ObjectSelectionTool
-
-static QPointF alignmentOffset(QRectF &r, Alignment alignment)
-{
-    switch (alignment) {
-    case TopLeft:       break;
-    case Top:           return QPointF(r.width() / 2, 0);
-    case TopRight:      return QPointF(r.width(), 0);
-    case Left:          return QPointF(0, r.height() / 2);
-    case Center:        return QPointF(r.width() / 2, r.height() / 2);
-    case Right:         return QPointF(r.width(), r.height() / 2);
-    case BottomLeft:    return QPointF(0, r.height());
-    case Bottom:        return QPointF(r.width() / 2, r.height());
-    case BottomRight:   return QPointF(r.width(), r.height());
-    }
-    return QPointF();
-}
-
-// TODO: Check whether this function should be moved into MapObject::bounds
-static void align(QRectF &r, Alignment alignment)
-{
-    r.translate(-alignmentOffset(r, alignment));
-}
-
-/* This function returns the actual bounds of the object, as opposed to the
- * bounds of its visualization that the MapRenderer::boundingRect function
- * returns.
- */
-static QRectF objectBounds(const MapObject *object,
-                           const MapRenderer *renderer)
-{
-    if (!object->cell().isEmpty()) {
-        // Tile objects can have a tile offset, which is scaled along with the image
-        QSizeF imgSize;
-        QPoint tileOffset;
-
-        if (const Tile *tile = object->cell().tile()) {
-            imgSize = tile->size();
-            tileOffset = tile->offset();
-        } else {
-            imgSize = object->size();
-        }
-
-        const QPointF position = renderer->pixelToScreenCoords(object->position());
-        const QSizeF objectSize = object->size();
-        const qreal scaleX = imgSize.width() > 0 ? objectSize.width() / imgSize.width() : 0;
-        const qreal scaleY = imgSize.height() > 0 ? objectSize.height() / imgSize.height() : 0;
-
-        QRectF bounds(position.x() + (tileOffset.x() * scaleX),
-                      position.y() + (tileOffset.y() * scaleY),
-                      objectSize.width(),
-                      objectSize.height());
-
-        align(bounds, object->alignment());
-
-        return bounds;
-    } else {
-        switch (object->shape()) {
-        case MapObject::Ellipse:
-        case MapObject::Rectangle: {
-            QRectF bounds(object->bounds());
-            align(bounds, object->alignment());
-            QPolygonF screenPolygon = renderer->pixelToScreenCoords(bounds);
-            return screenPolygon.boundingRect();
-        }
-        case MapObject::Point:
-            return renderer->shape(object).boundingRect();
-        case MapObject::Polygon:
-        case MapObject::Polyline: {
-            // Alignment is irrelevant for polygon objects since they have no size
-            const QPointF &pos = object->position();
-            const QPolygonF polygon = object->polygon().translated(pos);
-            QPolygonF screenPolygon = renderer->pixelToScreenCoords(polygon);
-            return screenPolygon.boundingRect();
-        }
-        case MapObject::Text:
-            return renderer->boundingRect(object);
-        }
-    }
-
-    return QRectF();
-}
 
+static const qreal labelMargin = 2;
+static const qreal labelDistance = 4;
 
 static Preferences::ObjectLabelVisiblity objectLabelVisibility()
 {
@@ -145,7 +60,8 @@ public:
         setZValue(1); // makes sure outlines are above labels
     }
 
-    void syncWithMapObject(MapRenderer *renderer);
+    MapObject *mapObject() const { return mObject; }
+    void syncWithMapObject(const MapRenderer &renderer);
 
     QRectF boundingRect() const override;
     void paint(QPainter *painter,
@@ -164,10 +80,10 @@ private:
     int mOffset = 0;
 };
 
-void MapObjectOutline::syncWithMapObject(MapRenderer *renderer)
+void MapObjectOutline::syncWithMapObject(const MapRenderer &renderer)
 {
-    const QPointF pixelPos = renderer->pixelToScreenCoords(mObject->position());
-    QRectF bounds = objectBounds(mObject, renderer);
+    const QPointF pixelPos = renderer.pixelToScreenCoords(mObject->position());
+    QRectF bounds = mObject->screenBounds(renderer);
 
     bounds.translate(-pixelPos);
 
@@ -202,11 +118,7 @@ void MapObjectOutline::paint(QPainter *painter,
     painter->setPen(pen);
     painter->drawLines(lines, 4);
 
-#if QT_VERSION >= 0x050600
     const qreal devicePixelRatio = painter->device()->devicePixelRatioF();
-#else
-    const int devicePixelRatio = painter->device()->devicePixelRatio();
-#endif
     const qreal dashLength = std::ceil(Utils::dpiScaled(3) * devicePixelRatio);
 
     // Draw a black dashed line above the white line
@@ -236,14 +148,14 @@ public:
     MapObjectLabel(MapObject *object, QGraphicsItem *parent = nullptr)
         : QGraphicsItem(parent)
         , mObject(object)
-        , mColor(MapObjectItem::objectColor(mObject))
+        , mColor(mObject->effectiveColor())
     {
         setFlags(QGraphicsItem::ItemIgnoresTransformations |
                  QGraphicsItem::ItemIgnoresParentOpacity);
     }
 
     MapObject *mapObject() const { return mObject; }
-    void syncWithMapObject(MapRenderer *renderer);
+    void syncWithMapObject(const MapRenderer &renderer);
     void updateColor();
 
     QRectF boundingRect() const override;
@@ -253,11 +165,12 @@ public:
 
 private:
     QRectF mBoundingRect;
+    QPointF mTextPos;
     MapObject *mObject;
     QColor mColor;
 };
 
-void MapObjectLabel::syncWithMapObject(MapRenderer *renderer)
+void MapObjectLabel::syncWithMapObject(const MapRenderer &renderer)
 {
     const bool nameVisible = mObject->isVisible() && !mObject->name().isEmpty();
     setVisible(nameVisible);
@@ -267,18 +180,22 @@ void MapObjectLabel::syncWithMapObject(MapRenderer *renderer)
 
     const QFontMetricsF metrics(QGuiApplication::font());
     QRectF boundingRect = metrics.boundingRect(mObject->name());
-    boundingRect.translate(-boundingRect.width() / 2, -labelDistance);
-    boundingRect.adjust(-labelMargin*2, -labelMargin, labelMargin*2, labelMargin);
 
-    QPointF pixelPos = renderer->pixelToScreenCoords(mObject->position());
-    QRectF bounds = objectBounds(mObject, renderer);
+    const qreal margin = Utils::dpiScaled(labelMargin);
+    const qreal distance = Utils::dpiScaled(labelDistance);
+    const qreal textY = -boundingRect.bottom() - margin - distance;
+
+    boundingRect.translate(-boundingRect.width() / 2, textY);
+
+    mTextPos = QPointF(boundingRect.left(), textY);
+
+    boundingRect.adjust(-margin*2, -margin, margin*2, margin);
+
+    QPointF pixelPos = renderer.pixelToScreenCoords(mObject->position());
+    QRectF bounds = mObject->screenBounds(renderer);
 
     // Adjust the bounding box for object rotation
-    QTransform transform;
-    transform.translate(pixelPos.x(), pixelPos.y());
-    transform.rotate(mObject->rotation());
-    transform.translate(-pixelPos.x(), -pixelPos.y());
-    bounds = transform.mapRect(bounds);
+    bounds = rotateAt(pixelPos, mObject->rotation()).mapRect(bounds);
 
     // Center the object name on the object bounding box
     QPointF pos((bounds.left() + bounds.right()) / 2, bounds.top());
@@ -295,7 +212,7 @@ void MapObjectLabel::syncWithMapObject(MapRenderer *renderer)
 
 void MapObjectLabel::updateColor()
 {
-    QColor color = MapObjectItem::objectColor(mObject);
+    const QColor color = mObject->effectiveColor();
     if (mColor != color) {
         mColor = color;
         update();
@@ -318,13 +235,11 @@ void MapObjectLabel::paint(QPainter *painter,
     painter->setBrush(mColor);
     painter->drawRoundedRect(mBoundingRect, 4, 4);
 
-    QPointF textPos(-(mBoundingRect.width() - labelMargin*4) / 2, -labelDistance);
-
     painter->drawRoundedRect(mBoundingRect, 4, 4);
     painter->setPen(Qt::black);
-    painter->drawText(textPos + QPointF(1,1), mObject->name());
+    painter->drawText(mTextPos + QPointF(1,1), mObject->name());
     painter->setPen(Qt::white);
-    painter->drawText(textPos, mObject->name());
+    painter->drawText(mTextPos, mObject->name());
 }
 
 
@@ -335,6 +250,9 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
 {
     setFlag(QGraphicsItem::ItemHasNoContents);
 
+    connect(mapDocument, &Document::changed,
+            this, &ObjectSelectionItem::changeEvent);
+
     connect(mapDocument, &MapDocument::selectedObjectsChanged,
             this, &ObjectSelectionItem::selectedObjectsChanged);
 
@@ -347,17 +265,11 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
     connect(mapDocument, &MapDocument::layerAboutToBeRemoved,
             this, &ObjectSelectionItem::layerAboutToBeRemoved);
 
-    connect(mapDocument, &MapDocument::layerChanged,
-            this, &ObjectSelectionItem::layerChanged);
+    connect(mapDocument, &MapDocument::hoveredMapObjectChanged,
+            this, &ObjectSelectionItem::hoveredMapObjectChanged);
 
-    connect(mapDocument, &MapDocument::objectsChanged,
-            this, &ObjectSelectionItem::syncOverlayItems);
-
-    connect(mapDocument, &MapDocument::objectsAdded,
-            this, &ObjectSelectionItem::objectsAdded);
-
-    connect(mapDocument, &MapDocument::objectsRemoved,
-            this, &ObjectSelectionItem::objectsRemoved);
+    connect(mapDocument, &MapDocument::tilesetTileOffsetChanged,
+            this, &ObjectSelectionItem::tilesetTileOffsetChanged);
 
     connect(mapDocument, &MapDocument::tileTypeChanged,
             this, &ObjectSelectionItem::tileTypeChanged);
@@ -374,12 +286,76 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
         addRemoveObjectLabels();
 }
 
+ObjectSelectionItem::~ObjectSelectionItem()
+{
+}
+
+void ObjectSelectionItem::changeEvent(const ChangeEvent &event)
+{
+    switch (event.type) {
+    case ChangeEvent::LayerChanged:
+        layerChanged(static_cast<const LayerChangeEvent&>(event).layer);
+        break;
+    case ChangeEvent::MapObjectsChanged:
+        syncOverlayItems(static_cast<const MapObjectsChangeEvent&>(event).mapObjects);
+        break;
+    case ChangeEvent::MapObjectsAdded:
+        objectsAdded(static_cast<const MapObjectsEvent&>(event).mapObjects);
+        break;
+    case ChangeEvent::MapObjectsAboutToBeRemoved:
+        objectsAboutToBeRemoved(static_cast<const MapObjectsEvent&>(event).mapObjects);
+        break;
+    case ChangeEvent::ObjectGroupChanged:
+        if (static_cast<const ObjectGroupChangeEvent&>(event).properties & ObjectGroupChangeEvent::ColorProperty)
+            updateObjectLabelColors();
+        break;
+    default:
+        break;
+    }
+}
+
 void ObjectSelectionItem::selectedObjectsChanged()
 {
     addRemoveObjectLabels();
     addRemoveObjectOutlines();
 }
 
+void ObjectSelectionItem::hoveredMapObjectChanged(MapObject *object,
+                                                  MapObject *previous)
+{
+    Preferences *prefs = Preferences::instance();
+    auto visibility = prefs->objectLabelVisibility();
+
+    if (visibility != Preferences::AllObjectLabels) {
+        bool labelForHoveredObject = prefs->labelForHoveredObject();
+
+        // Make sure any newly hovered object has a label
+        if (object && labelForHoveredObject && !mObjectLabels.contains(object)) {
+            MapObjectLabel *labelItem = new MapObjectLabel(object, this);
+            labelItem->syncWithMapObject(*mMapDocument->renderer());
+            mObjectLabels.insert(object, labelItem);
+        }
+
+        // Maybe remove the label from the previous object
+        if (MapObjectLabel *label = mObjectLabels.value(previous)) {
+            if (!(visibility == Preferences::SelectedObjectLabels &&
+                  mMapDocument->selectedObjects().contains(previous))) {
+                delete label;
+                mObjectLabels.remove(previous);
+            }
+        }
+    }
+
+    if (object && prefs->highlightHoveredObject()) {
+        mHoveredMapObjectItem = std::make_unique<MapObjectItem>(object, mMapDocument, this);
+        mHoveredMapObjectItem->setEnabled(false);
+        mHoveredMapObjectItem->setIsHoverIndicator(true);
+        mHoveredMapObjectItem->setZValue(-1.0);     // show below selection outlines
+    } else {
+        mHoveredMapObjectItem.reset();
+    }
+}
+
 void ObjectSelectionItem::mapChanged()
 {
     syncOverlayItems(mMapDocument->selectedObjects());
@@ -394,7 +370,7 @@ void ObjectSelectionItem::layerAdded(Layer *layer)
     // The layer may already have objects, for example when the addition is the
     // undo of a removal.
     if (objectLabelVisibility() == Preferences::AllObjectLabels) {
-        MapRenderer *renderer = mMapDocument->renderer();
+        const MapRenderer &renderer = *mMapDocument->renderer();
 
         for (MapObject *object : *objectGroup) {
             Q_ASSERT(!mObjectLabels.contains(object));
@@ -460,26 +436,28 @@ void ObjectSelectionItem::layerChanged(Layer *layer)
 
 void ObjectSelectionItem::syncOverlayItems(const QList<MapObject*> &objects)
 {
-    MapRenderer *renderer = mMapDocument->renderer();
+    const MapRenderer &renderer = *mMapDocument->renderer();
 
     for (MapObject *object : objects) {
         if (MapObjectOutline *outlineItem = mObjectOutlines.value(object))
             outlineItem->syncWithMapObject(renderer);
         if (MapObjectLabel *labelItem = mObjectLabels.value(object))
             labelItem->syncWithMapObject(renderer);
+        if (mHoveredMapObjectItem && mHoveredMapObjectItem->mapObject() == object)
+            mHoveredMapObjectItem->syncWithMapObject();
     }
 }
 
 void ObjectSelectionItem::updateObjectLabelColors()
 {
-    for (MapObjectLabel *label : mObjectLabels)
+    for (MapObjectLabel *label : qAsConst(mObjectLabels))
         label->updateColor();
 }
 
 void ObjectSelectionItem::objectsAdded(const QList<MapObject *> &objects)
 {
     if (objectLabelVisibility() == Preferences::AllObjectLabels) {
-        MapRenderer *renderer = mMapDocument->renderer();
+        const MapRenderer &renderer = *mMapDocument->renderer();
 
         for (MapObject *object : objects) {
             Q_ASSERT(!mObjectLabels.contains(object));
@@ -491,16 +469,33 @@ void ObjectSelectionItem::objectsAdded(const QList<MapObject *> &objects)
     }
 }
 
-void ObjectSelectionItem::objectsRemoved(const QList<MapObject *> &objects)
+void ObjectSelectionItem::objectsAboutToBeRemoved(const QList<MapObject *> &objects)
 {
     if (objectLabelVisibility() == Preferences::AllObjectLabels)
         for (MapObject *object : objects)
             delete mObjectLabels.take(object);
 }
 
+void ObjectSelectionItem::tilesetTileOffsetChanged(Tileset *tileset)
+{
+    // Tile offset affects the position of selection outlines and labels
+    const MapRenderer &renderer = *mMapDocument->renderer();
+
+    for (MapObjectLabel *label : qAsConst(mObjectLabels))
+        if (label->mapObject()->cell().tileset() == tileset)
+            label->syncWithMapObject(renderer);
+
+    for (MapObjectOutline *outline : qAsConst(mObjectOutlines))
+        if (outline->mapObject()->cell().tileset() == tileset)
+            outline->syncWithMapObject(renderer);
+
+    if (mHoveredMapObjectItem && mHoveredMapObjectItem->mapObject()->cell().tileset() == tileset)
+        mHoveredMapObjectItem->syncWithMapObject();
+}
+
 void ObjectSelectionItem::tileTypeChanged(Tile *tile)
 {
-    for (MapObjectLabel *label : mObjectLabels) {
+    for (MapObjectLabel *label : qAsConst(mObjectLabels)) {
         MapObject *object = label->mapObject();
         if (object->type().isEmpty()) {
             const auto &cell = object->cell();
@@ -518,9 +513,9 @@ void ObjectSelectionItem::objectLabelVisibilityChanged()
 void ObjectSelectionItem::addRemoveObjectLabels()
 {
     QHash<MapObject*, MapObjectLabel*> labelItems;
-    MapRenderer *renderer = mMapDocument->renderer();
+    const MapRenderer &renderer = *mMapDocument->renderer();
 
-    auto ensureLabel = [this,&labelItems,renderer] (MapObject *object) {
+    auto ensureLabel = [&] (MapObject *object) {
         if (labelItems.contains(object))
             return;
 
@@ -533,6 +528,11 @@ void ObjectSelectionItem::addRemoveObjectLabels()
         labelItems.insert(object, labelItem);
     };
 
+    Preferences *prefs = Preferences::instance();
+    if (prefs->labelForHoveredObject())
+        if (MapObject *object = mMapDocument->hoveredMapObject())
+            ensureLabel(object);
+
     switch (objectLabelVisibility()) {
     case Preferences::AllObjectLabels: {
         LayerIterator iterator(mMapDocument->map());
@@ -564,7 +564,7 @@ void ObjectSelectionItem::addRemoveObjectLabels()
 void ObjectSelectionItem::addRemoveObjectOutlines()
 {
     QHash<MapObject*, MapObjectOutline*> outlineItems;
-    MapRenderer *renderer = mMapDocument->renderer();
+    const MapRenderer &renderer = *mMapDocument->renderer();
 
     for (MapObject *mapObject : mMapDocument->selectedObjects()) {
         MapObjectOutline *outlineItem = mObjectOutlines.take(mapObject);
@@ -579,5 +579,4 @@ void ObjectSelectionItem::addRemoveObjectOutlines()
     mObjectOutlines.swap(outlineItems);
 }
 
-} // namespace Internal
 } // namespace Tiled
