@@ -20,19 +20,23 @@
 
 #pragma once
 
+#include "changeevents.h"
+
 #include <QGraphicsObject>
 #include <QHash>
 
+#include <memory>
+
 namespace Tiled {
 
 class GroupLayer;
 class Layer;
 class MapObject;
 class Tile;
-
-namespace Internal {
+class Tileset;
 
 class MapDocument;
+class MapObjectItem;
 class MapObjectLabel;
 class MapObjectOutline;
 
@@ -49,14 +53,16 @@ class ObjectSelectionItem : public QGraphicsObject
 public:
     ObjectSelectionItem(MapDocument *mapDocument,
                         QGraphicsItem *parent = nullptr);
+    ~ObjectSelectionItem() override;
 
     // QGraphicsItem interface
     QRectF boundingRect() const override { return QRectF(); }
     void paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *) override {}
 
-
-private slots:
+private:
+    void changeEvent(const ChangeEvent &event);
     void selectedObjectsChanged();
+    void hoveredMapObjectChanged(MapObject *object, MapObject *previous);
     void mapChanged();
     void layerAdded(Layer *layer);
     void layerAboutToBeRemoved(GroupLayer *parentLayer, int index);
@@ -64,19 +70,19 @@ private slots:
     void syncOverlayItems(const QList<MapObject *> &objects);
     void updateObjectLabelColors();
     void objectsAdded(const QList<MapObject*> &objects);
-    void objectsRemoved(const QList<MapObject*> &objects);
+    void objectsAboutToBeRemoved(const QList<MapObject*> &objects);
+    void tilesetTileOffsetChanged(Tileset *tileset);
     void tileTypeChanged(Tile *tile);
 
     void objectLabelVisibilityChanged();
 
-private:
     void addRemoveObjectLabels();
     void addRemoveObjectOutlines();
 
     MapDocument *mMapDocument;
     QHash<MapObject*, MapObjectLabel*> mObjectLabels;
     QHash<MapObject*, MapObjectOutline*> mObjectOutlines;
+    std::unique_ptr<MapObjectItem> mHoveredMapObjectItem;
 };
 
-} // namespace Internal
 } // namespace Tiled
