@@ -30,24 +30,18 @@
 #include <QMimeData>
 
 namespace Tiled {
-namespace Internal {
 
 ObjectTemplateModel::ObjectTemplateModel(QObject *parent):
     QFileSystemModel(parent)
 {
-    QStringList nameFilters;
-
-    for (ObjectTemplateFormat *format : PluginManager::objects<ObjectTemplateFormat>()) {
-        if (!(format->capabilities() & FileFormat::Read))
-            continue;
-
-        const QString filter = format->nameFilter();
-        nameFilters.append(Utils::cleanFilterList(filter));
-    }
-
     setFilter(QDir::AllDirs | QDir::Files | QDir::NoDotAndDotDot);
-    setNameFilters(nameFilters);
     setNameFilterDisables(false); // hide filtered files
+    updateNameFilters();
+
+    connect(PluginManager::instance(), &PluginManager::objectAdded,
+            this, &ObjectTemplateModel::pluginObjectAddedOrRemoved);
+    connect(PluginManager::instance(), &PluginManager::objectRemoved,
+            this, &ObjectTemplateModel::pluginObjectAddedOrRemoved);
 }
 
 int ObjectTemplateModel::columnCount(const QModelIndex &parent) const
@@ -104,5 +98,27 @@ QMimeData *ObjectTemplateModel::mimeData(const QModelIndexList &indexes) const
     return mimeData;
 }
 
-} // namespace Internal
+void ObjectTemplateModel::pluginObjectAddedOrRemoved(QObject *object)
+{
+    if (auto format = qobject_cast<ObjectTemplateFormat*>(object))
+        if (format->capabilities() & FileFormat::Read)
+            updateNameFilters();
+}
+
+void ObjectTemplateModel::updateNameFilters()
+{
+    QStringList nameFilters;
+
+    const auto formats = PluginManager::objects<ObjectTemplateFormat>();
+    for (ObjectTemplateFormat *format : formats) {
+        if (!(format->capabilities() & FileFormat::Read))
+            continue;
+
+        const QString filter = format->nameFilter();
+        nameFilters.append(Utils::cleanFilterList(filter));
+    }
+
+    setNameFilters(nameFilters);
+}
+
 } // namespace Tiled
