@@ -27,8 +27,6 @@ namespace Tiled {
 
 class ObjectTemplate;
 
-namespace Internal {
-
 class ObjectTemplateModel : public QFileSystemModel
 {
     Q_OBJECT
@@ -43,7 +41,11 @@ public:
     Qt::ItemFlags flags(const QModelIndex &index) const override;
     QStringList mimeTypes() const override;
     QMimeData *mimeData(const QModelIndexList &indexes) const override;
+
+private:
+    void pluginObjectAddedOrRemoved(QObject *object);
+
+    void updateNameFilters();
 };
 
-} // namespace Internal
 } // namespace Tiled
