@@ -25,6 +25,7 @@
 #include <QDate>
 #include <QObject>
 
+#include "filesystemwatcher.h"
 #include "map.h"
 #include "objecttypes.h"
 
@@ -246,6 +247,10 @@ private:
     int intValue(const char *key, int defaultValue) const;
     qreal realValue(const char *key, qreal defaultValue) const;
 
+    void objectTypesFileChangedOnDisk();
+
+    FileSystemWatcher mWatcher;
+
     QSettings *mSettings;
 
     bool mShowGrid;
