@@ -18,15 +18,13 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILED_INTERNAL_REPLACETILESET_H
-#define TILED_INTERNAL_REPLACETILESET_H
+#pragma once
 
 #include "tileset.h"
 
 #include <QUndoCommand>
 
 namespace Tiled {
-namespace Internal {
 
 class MapDocument;
 
@@ -48,7 +46,4 @@ private:
     SharedTileset mTileset;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // TILED_INTERNAL_REPLACETILESET_H
