@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef RESIZELAYER_H
-#define RESIZELAYER_H
+#pragma once
 
 #include <QPoint>
 #include <QSize>
@@ -30,8 +29,6 @@ namespace Tiled {
 class Layer;
 class TileLayer;
 
-namespace Internal {
-
 class MapDocument;
 
 /**
@@ -46,8 +43,9 @@ public:
      */
     ResizeTileLayer(MapDocument *mapDocument,
                     TileLayer *layer,
-                    const QSize &size,
-                    const QPoint &offset);
+                    QSize size,
+                    QPoint offset,
+                    QUndoCommand *parent = nullptr);
 
     ~ResizeTileLayer();
 
@@ -55,15 +53,10 @@ public:
     void redo() override;
 
 private:
-    Layer *swapLayer(Layer *layer);
-
     MapDocument *mMapDocument;
-    int mIndex;
+    bool mDone;
     TileLayer *mOriginalLayer;
     TileLayer *mResizedLayer;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // RESIZELAYER_H
