@@ -18,13 +18,11 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef SELECTIONRECTANGLE_H
-#define SELECTIONRECTANGLE_H
+#pragma once
 
 #include <QGraphicsItem>
 
 namespace Tiled {
-namespace Internal {
 
 /**
  * The rectangle used for indicating the dragged area when selecting items.
@@ -45,7 +43,4 @@ private:
     QRectF mRectangle;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // SELECTIONRECTANGLE_H
