@@ -18,14 +18,12 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILED_INTERNAL_STYLEHELPER_H
-#define TILED_INTERNAL_STYLEHELPER_H
+#pragma once
 
 #include <QPalette>
 #include <QString>
 
 namespace Tiled {
-namespace Internal {
 
 class StyleHelper : public QObject
 {
@@ -52,7 +50,4 @@ private:
     static StyleHelper *mInstance;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // TILED_INTERNAL_STYLEHELPER_H
