@@ -34,8 +34,6 @@ class Tile;
 class TileAnimationDriver;
 class Tileset;
 
-namespace Internal {
-
 class FrameListModel;
 class TilesetDocument;
 
@@ -62,32 +60,34 @@ protected:
     void showEvent(QShowEvent *) override;
     void hideEvent(QHideEvent *) override;
 
-private slots:
+private:
     void framesEdited();
     void tileAnimationChanged(Tile *tile);
     void currentObjectChanged(Object *object);
 
     void addFrameForTileAt(const QModelIndex &index);
 
+    void setFrameTime();
+    void setDefaultFrameTime(int duration);
     void undo();
     void redo();
     void delete_();
 
     void advancePreviewAnimation(int ms);
     void resetPreview();
+    bool updatePreviewPixmap();
 
-private:
     Ui::TileAnimationEditor *mUi;
 
     TilesetDocument *mTilesetDocument;
     Tile *mTile;
     FrameListModel *mFrameListModel;
     bool mApplyingChanges;
+    bool mSuppressUndo;
 
     TileAnimationDriver *mPreviewAnimationDriver;
     int mPreviewFrameIndex;
     int mPreviewUnusedTime;
 };
 
-} // namespace Internal
 } // namespace Tiled
