@@ -18,23 +18,26 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILECOLLISIONDOCK_H
-#define TILECOLLISIONDOCK_H
+#pragma once
 
 #include "clipboardmanager.h"
+#include "mapdocument.h"
 
 #include <QDockWidget>
 
+class QSplitter;
+
 namespace Tiled {
 
 class Object;
 class Tile;
-
-namespace Internal {
+class Tileset;
 
 class AbstractTool;
+class EditableMapObject;
 class MapScene;
 class MapView;
+class ObjectsView;
 class TilesetDocument;
 class ToolManager;
 
@@ -42,21 +45,42 @@ class TileCollisionDock : public QDockWidget
 {
     Q_OBJECT
 
+    Q_PROPERTY(QList<QObject*> selectedObjects READ selectedObjectsForScript WRITE setSelectedObjectsFromScript)
+    Q_PROPERTY(Tiled::MapView *view READ mapView)
+
+public:
     enum Operation {
         Cut,
         Delete
     };
 
-public:
+    enum ObjectsViewVisibility {
+        Hidden,
+        ShowRight,
+        ShowBottom
+    };
+    Q_ENUM(ObjectsViewVisibility)
+
     explicit TileCollisionDock(QWidget *parent = nullptr);
     ~TileCollisionDock() override;
 
+    void saveState();
+    void restoreState();
+
     void setTilesetDocument(TilesetDocument *tilesetDocument);
 
     MapDocument *dummyMapDocument() const;
+    MapView *mapView() const;
+
+    ToolManager *toolManager() const;
 
     bool hasSelectedObjects() const;
 
+    QList<QObject*> selectedObjectsForScript() const;
+    void setSelectedObjectsFromScript(const QList<QObject*> &selectedObjects);
+
+    Q_INVOKABLE void focusObject(Tiled::EditableMapObject *object);
+
 signals:
     void dummyMapDocumentChanged(MapDocument *mapDocument);
     void hasSelectedObjectsChanged();
@@ -75,31 +99,65 @@ public slots:
 protected:
     void changeEvent(QEvent *e) override;
 
-private slots:
-    void setSelectedTool(AbstractTool*);
+private:
     void applyChanges();
+    void documentChanged(const ChangeEvent &change);
     void tileObjectGroupChanged(Tile*);
+    void tilesetTileOffsetChanged(Tileset *tileset);
 
     void selectedObjectsChanged();
     void setHasSelectedObjects(bool hasSelectedObjects);
 
-private:
+    void selectAll();
+
+    void duplicateObjects();
+    void removeObjects();
+    void moveObjectsUp();
+    void moveObjectsDown();
+    void objectProperties();
+
+    void setObjectsViewVisibility(ObjectsViewVisibility);
+
+    MapObject *clonedObjectForScriptObject(EditableMapObject *scriptObject);
+
     void retranslateUi();
 
-    Tile *mTile;
-    TilesetDocument *mTilesetDocument;
-    MapDocument *mDummyMapDocument;
+    Tile *mTile = nullptr;
+    TilesetDocument *mTilesetDocument = nullptr;
+    MapDocumentPtr mDummyMapDocument;
     MapScene *mMapScene;
     MapView *mMapView;
+    ObjectsView *mObjectsView;
+    QWidget *mObjectsWidget;
+    QSplitter *mObjectsViewSplitter;
+    QAction *mObjectsViewHiddenAction;
+    QAction *mObjectsViewShowRightAction;
+    QAction *mObjectsViewShowBottomAction;
     ToolManager *mToolManager;
-    bool mApplyingChanges;
-    bool mSynchronizing;
-    bool mHasSelectedObjects;
+    QAction *mActionDuplicateObjects;
+    QAction *mActionRemoveObjects;
+    QAction *mActionMoveUp;
+    QAction *mActionMoveDown;
+    QAction *mActionObjectProperties;
+    bool mApplyingChanges = false;
+    bool mSynchronizing = false;
+    bool mHasSelectedObjects = false;
+    ObjectsViewVisibility mObjectsViewVisibility = Hidden;
 };
 
 inline MapDocument *TileCollisionDock::dummyMapDocument() const
 {
-    return mDummyMapDocument;
+    return mDummyMapDocument.data();
+}
+
+inline MapView *TileCollisionDock::mapView() const
+{
+    return mMapView;
+}
+
+inline ToolManager *TileCollisionDock::toolManager() const
+{
+    return mToolManager;
 }
 
 inline bool TileCollisionDock::hasSelectedObjects() const
@@ -107,7 +165,6 @@ inline bool TileCollisionDock::hasSelectedObjects() const
     return mHasSelectedObjects;
 }
 
-} // namespace Internal
 } // namespace Tiled
 
-#endif // TILECOLLISIONDOCK_H
+Q_DECLARE_METATYPE(Tiled::TileCollisionDock*)
