@@ -20,6 +20,7 @@
 
 #include "tileselectionitem.h"
 
+#include "changeevents.h"
 #include "grouplayer.h"
 #include "map.h"
 #include "mapdocument.h"
@@ -31,18 +32,19 @@
 #include <QStyleOptionGraphicsItem>
 
 using namespace Tiled;
-using namespace Tiled::Internal;
 
-TileSelectionItem::TileSelectionItem(MapDocument *mapDocument)
-    : mMapDocument(mapDocument)
+TileSelectionItem::TileSelectionItem(MapDocument *mapDocument,
+                                     QGraphicsItem *parent)
+    : QGraphicsObject(parent)
+    , mMapDocument(mapDocument)
 {
     setFlag(QGraphicsItem::ItemUsesExtendedStyleOption);
 
-    connect(mMapDocument, &MapDocument::selectedAreaChanged,
+    connect(mapDocument, &MapDocument::changed,
+            this, &TileSelectionItem::documentChanged);
+    connect(mapDocument, &MapDocument::selectedAreaChanged,
             this, &TileSelectionItem::selectionChanged);
-    connect(mapDocument, &MapDocument::layerChanged,
-            this, &TileSelectionItem::layerChanged);
-    connect(mMapDocument, &MapDocument::currentLayerChanged,
+    connect(mapDocument, &MapDocument::currentLayerChanged,
             this, &TileSelectionItem::currentLayerChanged);
 
     updateBoundingRect();
@@ -66,6 +68,22 @@ void TileSelectionItem::paint(QPainter *painter,
                                 option->exposedRect);
 }
 
+void TileSelectionItem::documentChanged(const ChangeEvent &change)
+{
+    switch (change.type) {
+    case ChangeEvent::LayerChanged: {
+        const auto &layerChange = static_cast<const LayerChangeEvent&>(change);
+        if (layerChange.properties & LayerChangeEvent::OffsetProperty)
+            if (auto currentLayer = mMapDocument->currentLayer())
+                if (currentLayer->isParentOrSelf(layerChange.layer))
+                    setPos(currentLayer->totalOffset());
+        break;
+    }
+    default:
+        break;
+    }
+}
+
 void TileSelectionItem::selectionChanged(const QRegion &newSelection,
                                          const QRegion &oldSelection)
 {
@@ -77,13 +95,6 @@ void TileSelectionItem::selectionChanged(const QRegion &newSelection,
     update(mMapDocument->renderer()->boundingRect(changedArea));
 }
 
-void TileSelectionItem::layerChanged(Layer *layer)
-{
-    if (auto currentLayer = mMapDocument->currentLayer())
-        if (currentLayer->isParentOrSelf(layer))
-            setPos(currentLayer->totalOffset());
-}
-
 void TileSelectionItem::currentLayerChanged(Layer *layer)
 {
     if (layer)
