@@ -33,8 +33,6 @@ namespace Tiled {
 
 class Tileset;
 
-namespace Internal {
-
 class TilesetDocument;
 
 class RenameTileset : public QUndoCommand
@@ -83,7 +81,7 @@ struct TilesetParameters
 
     bool operator != (const TilesetParameters &other) const;
 
-    QString imageSource;
+    QUrl imageSource;
     QColor transparentColor;
     QSize tileSize;
     int tileSpacing;
@@ -175,5 +173,4 @@ private:
     QSize mGridSize;
 };
 
-} // namespace Internal
 } // namespace Tiled
