@@ -34,7 +34,6 @@ class QActionGroup;
 class QComboBox;
 class QMenu;
 class QModelIndex;
-class QSignalMapper;
 class QStackedWidget;
 class QTabBar;
 class QToolBar;
@@ -47,14 +46,13 @@ class Tile;
 class TileLayer;
 class Tileset;
 
-namespace Internal {
-
 class Document;
+class EditableTileset;
 class MapDocument;
+class TileStamp;
 class TilesetDocument;
 class TilesetDocumentsFilterModel;
 class TilesetView;
-class TileStamp;
 class Zoomable;
 
 /**
@@ -65,13 +63,12 @@ class TilesetDock : public QDockWidget
 {
     Q_OBJECT
 
+    Q_PROPERTY(Tiled::EditableTileset *currentTileset READ currentEditableTileset WRITE setCurrentEditableTileset)
+
 public:
-    /**
-     * Constructor.
-     */
     TilesetDock(QWidget *parent = nullptr);
 
-    ~TilesetDock();
+    ~TilesetDock() override;
 
     /**
      * Sets the map for which the tilesets should be displayed.
@@ -83,8 +80,17 @@ public:
      */
     Tile *currentTile() const { return mCurrentTile; }
 
+    void setCurrentTileset(const SharedTileset &tileset);
+    SharedTileset currentTileset() const;
+
+    void setCurrentEditableTileset(EditableTileset *tileset);
+    EditableTileset *currentEditableTileset() const;
+
     void selectTilesInStamp(const TileStamp &);
 
+    QAction *actionSelectNextTileset() const { return mSelectNextTileset; }
+    QAction *actionSelectPreviousTileset() const { return mSelectPreviousTileset; }
+
 signals:
     /**
      * Emitted when the current tile changed.
@@ -108,7 +114,7 @@ protected:
     void dragEnterEvent(QDragEnterEvent *) override;
     void dropEvent(QDropEvent *) override;
 
-private slots:
+private:
     void currentTilesetChanged();
     void selectionChanged();
     void currentChanged(const QModelIndex &index);
@@ -124,7 +130,7 @@ private slots:
     void tileAnimationChanged(Tile *tile);
 
     void removeTileset();
-    void removeTileset(int index);
+    void removeTilesetAt(int index);
 
     void newTileset();
     void editTileset();
@@ -135,7 +141,6 @@ private slots:
 
     void swapTiles(Tile *tileA, Tile *tileB);
 
-private:
     void setCurrentTile(Tile *tile);
     void setCurrentTiles(TileLayer *tiles);
     void retranslateUi();
@@ -149,7 +154,6 @@ private:
     void onTabMoved(int from, int to);
     void tabContextMenuRequested(const QPoint &pos);
 
-    Tileset *currentTileset() const;
     TilesetView *currentTilesetView() const;
     TilesetView *tilesetViewAt(int index) const;
 
@@ -178,11 +182,13 @@ private:
     QAction *mExportTileset;
     QAction *mEditTileset;
     QAction *mDeleteTileset;
+    QAction *mSelectNextTileset;
+    QAction *mSelectPreviousTileset;
+    QAction *mDynamicWrappingToggle;
 
     QToolButton *mTilesetMenuButton;
     QMenu *mTilesetMenu; //opens on click of mTilesetMenu
     QActionGroup *mTilesetActionGroup;
-    QSignalMapper *mTilesetMenuMapper; //needed due to dynamic content
 
     QComboBox *mZoomComboBox;
 
@@ -190,5 +196,4 @@ private:
     bool mSynchronizingSelection;
 };
 
-} // namespace Internal
 } // namespace Tiled
