@@ -21,23 +21,28 @@
 #pragma once
 
 #include "document.h"
+#include "editabletileset.h"
 #include "tileset.h"
 #include "tilesetformat.h"
 
 #include <QList>
+#include <QMap>
 
 #include <memory>
 #include <unordered_map>
 
 namespace Tiled {
 
-namespace Internal {
+class ObjectGroup;
 
 class MapDocument;
+class TilesetDocument;
 class TilesetTerrainModel;
 class TilesetWangSetModel;
 class WangColorModel;
 
+using TilesetDocumentPtr = QSharedPointer<TilesetDocument>;
+
 /**
  * Represents an editable tileset.
  */
@@ -46,9 +51,11 @@ class TilesetDocument : public Document
     Q_OBJECT
 
 public:
-    TilesetDocument(const SharedTileset &tileset, const QString &fileName = QString());
+    TilesetDocument(const SharedTileset &tileset);
     ~TilesetDocument() override;
 
+    TilesetDocumentPtr sharedFromThis() { return qSharedPointerCast<TilesetDocument>(Document::sharedFromThis()); }
+
     bool save(const QString &fileName, QString *error = nullptr) override;
 
     bool canReload() const;
@@ -58,13 +65,16 @@ public:
      * Loads a tileset and returns a TilesetDocument instance on success.
      * Returns null on error and sets the \a error message.
      */
-    static TilesetDocument *load(const QString &fileName,
-                                 TilesetFormat *format,
-                                 QString *error = nullptr);
+    static TilesetDocumentPtr load(const QString &fileName,
+                                   TilesetFormat *format,
+                                   QString *error = nullptr);
 
     FileFormat *writerFormat() const override;
     void setWriterFormat(TilesetFormat *format);
 
+    QString lastExportFileName() const override;
+    void setLastExportFileName(const QString &fileName) override;
+
     TilesetFormat *exportFormat() const override;
     void setExportFormat(FileFormat *format) override;
 
@@ -73,6 +83,8 @@ public:
     void swapTileset(SharedTileset &tileset);
     const SharedTileset &tileset() const;
 
+    EditableTileset *editable() override;
+
     bool isEmbedded() const;
     void setClean();
 
@@ -81,7 +93,7 @@ public:
     void removeMapDocument(MapDocument *mapDocument);
 
     void setTilesetName(const QString &name);
-    void setTilesetTileOffset(const QPoint &tileOffset);
+    void setTilesetTileOffset(QPoint tileOffset);
 
     void addTiles(const QList<Tile*> &tiles);
     void removeTiles(const QList<Tile*> &tiles);
@@ -98,6 +110,12 @@ public:
 
     void setTileType(Tile *tile, const QString &type);
     void setTileImage(Tile *tile, const QPixmap &image, const QUrl &source);
+    void setTileProbability(Tile *tile, qreal probability);
+    void swapTileObjectGroup(Tile *tile, std::unique_ptr<ObjectGroup> &objectGroup);
+
+    void checkIssues() override;
+
+    static TilesetDocument* findDocumentForTileset(const SharedTileset &tileset);
 
 signals:
     /**
@@ -109,6 +127,9 @@ signals:
      */
     void tilesetChanged(Tileset *tileset);
 
+    void tilesAdded(const QList<Tile*> &tiles);
+    void tilesRemoved(const QList<Tile*> &tiles);
+
     void tilesetNameChanged(Tileset *tileset);
     void tilesetTileOffsetChanged(Tileset *tileset);
 
@@ -143,7 +164,7 @@ signals:
      */
     void selectedTilesChanged();
 
-private slots:
+private:
     void onPropertyAdded(Object *object, const QString &name);
     void onPropertyRemoved(Object *object, const QString &name);
     void onPropertyChanged(Object *object, const QString &name);
@@ -152,17 +173,16 @@ private slots:
     void onTerrainRemoved(Terrain *terrain);
     void onWangSetRemoved(WangSet *wangSet);
 
-private:
     SharedTileset mTileset;
     QList<MapDocument*> mMapDocuments;
 
     TilesetTerrainModel *mTerrainModel;
     TilesetWangSetModel *mWangSetModel;
-    WangColorModel *mWangColorModel;
     std::unordered_map<WangSet*, std::unique_ptr<WangColorModel>> mWangColorModels;
 
     QList<Tile*> mSelectedTiles;
-    QPointer<TilesetFormat> mExportFormat;
+
+    static QMap<SharedTileset, TilesetDocument*> sTilesetToDocument;
 };
 
 
@@ -192,5 +212,4 @@ inline const QList<Tile *> &TilesetDocument::selectedTiles() const
     return mSelectedTiles;
 }
 
-} // namespace Internal
 } // namespace Tiled
