@@ -32,8 +32,6 @@ namespace Tiled {
 class Map;
 class TileLayer;
 
-namespace Internal {
-
 class MapDocument;
 class TileStamp;
 class TileStampModel;
@@ -77,13 +75,12 @@ private:
 
     void loadStamps();
 
-private slots:
+private:
     void stampAdded(TileStamp stamp);
     void stampRenamed(TileStamp stamp);
     void saveStamp(const TileStamp &stamp);
     void deleteStamp(const TileStamp &stamp);
 
-private:
     QVector<TileStamp> mQuickStamps;
     QMap<QString, TileStamp> mStampsByName;
     TileStampModel *mTileStampModel;
@@ -99,17 +96,17 @@ private:
  */
 inline QList<Qt::Key> TileStampManager::quickStampKeys()
 {
-    QList<Qt::Key> keys;
-    keys << Qt::Key_1
-         << Qt::Key_2
-         << Qt::Key_3
-         << Qt::Key_4
-         << Qt::Key_5
-         << Qt::Key_6
-         << Qt::Key_7
-         << Qt::Key_8
-         << Qt::Key_9;
-    return keys;
+    return {
+        Qt::Key_1,
+        Qt::Key_2,
+        Qt::Key_3,
+        Qt::Key_4,
+        Qt::Key_5,
+        Qt::Key_6,
+        Qt::Key_7,
+        Qt::Key_8,
+        Qt::Key_9
+    };
 }
 
 inline TileStampModel *TileStampManager::tileStampModel() const
@@ -117,5 +114,4 @@ inline TileStampModel *TileStampManager::tileStampModel() const
     return mTileStampModel;
 }
 
-} // namespace Tiled::Internal
 } // namespace Tiled
