@@ -22,12 +22,13 @@
 
 #include "mapformat.h"
 #include "tilesetformat.h"
+#include "objecttemplateformat.h"
 
 namespace Tiled {
 
 class Tileset;
+class MapObject;
 
-namespace Internal {
 
 /**
  * A reader and writer for Tiled's .tmx map format.
@@ -40,9 +41,9 @@ class TmxMapFormat : public MapFormat
 public:
     TmxMapFormat(QObject *parent = nullptr);
 
-    Map *read(const QString &fileName) override;
+    std::unique_ptr<Map> read(const QString &fileName) override;
 
-    bool write(const Map *map, const QString &fileName) override;
+    bool write(const Map *map, const QString &fileName, Options options) override;
 
     /**
      * Converts the given map to a utf8 byte array (in .tmx format). This is
@@ -59,9 +60,9 @@ public:
      *
      * @see toByteArray
      */
-    Map *fromByteArray(const QByteArray &data);
+    std::unique_ptr<Map> fromByteArray(const QByteArray &data);
 
-    QString nameFilter() const override { return tr("Tiled map files (*.tmx)"); }
+    QString nameFilter() const override { return tr("Tiled map files (*.tmx *.xml)"); }
 
     QString shortName() const override { return QLatin1String("tmx"); }
 
@@ -87,9 +88,9 @@ public:
 
     SharedTileset read(const QString &fileName) override;
 
-    bool write(const Tileset &tileset, const QString &fileName) override;
+    bool write(const Tileset &tileset, const QString &fileName, Options options) override;
 
-    QString nameFilter() const override { return tr("Tiled tileset files (*.tsx)"); }
+    QString nameFilter() const override { return tr("Tiled tileset files (*.tsx *.xml)"); }
 
     QString shortName() const override { return QLatin1String("tsx"); }
 
@@ -101,5 +102,31 @@ private:
     QString mError;
 };
 
-} // namespace Internal
+/**
+ * A reader and writer for Tiled's .tgx template format.
+ */
+class XmlObjectTemplateFormat : public ObjectTemplateFormat
+{
+    Q_OBJECT
+    Q_INTERFACES(Tiled::ObjectTemplateFormat)
+
+public:
+    XmlObjectTemplateFormat(QObject *parent = nullptr);
+
+    std::unique_ptr<ObjectTemplate> read(const QString &fileName) override;
+
+    bool write(const ObjectTemplate *objectTemplate, const QString &fileName) override;
+
+    QString nameFilter() const override { return tr("Tiled template files (*.tx)"); }
+
+    QString shortName() const override { return QLatin1String("tx"); }
+
+    bool supportsFile(const QString &fileName) const override;
+
+    QString errorString() const override { return mError; }
+
+private:
+    QString mError;
+};
+
 } // namespace Tiled
