@@ -21,6 +21,7 @@
 #include "toolmanager.h"
 
 #include "abstracttool.h"
+#include "actionmanager.h"
 #include "preferences.h"
 
 #include <QAction>
@@ -31,13 +32,6 @@ using namespace Tiled;
 ToolManager::ToolManager(QObject *parent)
     : QObject(parent)
     , mActionGroup(new QActionGroup(this))
-    , mSelectedTool(nullptr)
-    , mDisabledTool(nullptr)
-    , mPreviouslyDisabledTool(nullptr)
-    , mMapDocument(nullptr)
-    , mTile(nullptr)
-    , mObjectTemplate(nullptr)
-    , mSelectEnabledToolPending(false)
 {
     mActionGroup->setExclusive(true);
     connect(mActionGroup, &QActionGroup::triggered,
@@ -51,6 +45,15 @@ ToolManager::~ToolManager()
 {
 }
 
+/**
+ * Can be used to disable the registration of actions with the ActionManager.
+ * Should be called before any tools are registered.
+ */
+void ToolManager::setRegisterActions(bool enabled)
+{
+    mRegisterActions = enabled;
+}
+
 /**
  * Sets the MapDocument on which the registered tools will operate.
  */
@@ -81,17 +84,18 @@ QAction *ToolManager::registerTool(AbstractTool *tool)
 
     tool->setMapDocument(mMapDocument);
 
+    QString toolTip = tool->name();
+    QKeySequence shortcut = tool->shortcut();
+    if (!shortcut.isEmpty()) {
+        toolTip = QString(QLatin1String("%1 (%2)")).arg(toolTip,
+                                                        shortcut.toString());
+    }
+
     QAction *toolAction = new QAction(tool->icon(), tool->name(), this);
-    toolAction->setShortcut(tool->shortcut());
+    toolAction->setShortcut(shortcut);
     toolAction->setData(QVariant::fromValue<AbstractTool*>(tool));
     toolAction->setCheckable(true);
-    if (!tool->shortcut().isEmpty()) {
-        toolAction->setToolTip(
-                QString(QLatin1String("%1 (%2)")).arg(tool->name(),
-                                                      tool->shortcut().toString()));
-    } else {
-        toolAction->setToolTip(tool->name());
-    }
+    toolAction->setToolTip(toolTip);
     toolAction->setEnabled(tool->isEnabled());
 
     mActionGroup->addAction(toolAction);
@@ -99,6 +103,9 @@ QAction *ToolManager::registerTool(AbstractTool *tool)
     connect(tool, &AbstractTool::changed,
             this, &ToolManager::toolChanged);
 
+    connect(toolAction, &QAction::changed,
+            this, &ToolManager::toolActionChanged);
+
     connect(tool, &AbstractTool::enabledChanged,
             this, &ToolManager::toolEnabledChanged);
 
@@ -108,6 +115,9 @@ QAction *ToolManager::registerTool(AbstractTool *tool)
         toolAction->setChecked(true);
     }
 
+    if (mRegisterActions)
+        ActionManager::registerAction(toolAction, tool->id());
+
     return toolAction;
 }
 
@@ -115,6 +125,10 @@ void ToolManager::unregisterTool(AbstractTool *tool)
 {
     auto action = findAction(tool);
     Q_ASSERT(action);
+
+    if (mRegisterActions)
+        ActionManager::unregisterAction(action, tool->id());
+
     delete action;
 
     tool->disconnect(this);
@@ -180,15 +194,29 @@ void ToolManager::toolChanged()
         action->setText(tool->name());
         action->setIcon(tool->icon());
         action->setShortcut(tool->shortcut());
-        if (!tool->shortcut().isEmpty()) {
-            action->setToolTip(QStringLiteral("%1 (%2)").arg(tool->name(),
-                                                             tool->shortcut().toString()));
-        } else {
-            action->setToolTip(tool->name());
-        }
     }
 }
 
+void ToolManager::toolActionChanged()
+{
+    if (mUpdatingActionToolTip)
+        return;
+
+    auto action = static_cast<QAction*>(sender());
+
+    QString toolTip = action->text();
+    QKeySequence shortcut = action->shortcut();
+
+    if (!shortcut.isEmpty()) {
+        toolTip = QString(QLatin1String("%1 (%2)")).arg(toolTip,
+                                                        shortcut.toString());
+    }
+
+    mUpdatingActionToolTip = true;
+    action->setToolTip(toolTip);
+    mUpdatingActionToolTip = false;
+}
+
 void ToolManager::retranslateTools()
 {
     // Allow the tools to adapt to the new language
@@ -197,15 +225,7 @@ void ToolManager::retranslateTools()
         AbstractTool *tool = action->data().value<AbstractTool*>();
         tool->languageChanged();
 
-        // Update the text, shortcut and tooltip of the action
         action->setText(tool->name());
-        action->setShortcut(tool->shortcut());
-        if (!tool->shortcut().isEmpty()) {
-            action->setToolTip(QStringLiteral("%1 (%2)").arg(tool->name(),
-                                                             tool->shortcut().toString()));
-        } else {
-            action->setToolTip(tool->name());
-        }
     }
 }
 
