@@ -20,16 +20,38 @@
 
 #pragma once
 
+class QUndoCommand;
+
+namespace Tiled {
+
 /**
  * These undo command IDs are used by Qt to determine whether two undo commands
  * can be merged.
  */
 enum UndoCommands {
-    Cmd_EraseTiles,
-    Cmd_PaintTileLayer,
     Cmd_ChangeLayerOffset,
     Cmd_ChangeLayerOpacity,
     Cmd_ChangeTileTerrain,
     Cmd_ChangeTileWangId,
-    Cmd_ChangeTilesetTileOffset
+    Cmd_ChangeTilesetTileOffset,
+    Cmd_EraseTiles,
+    Cmd_PaintTileLayer
 };
+
+/**
+ * Interface to be implemented by undo commands that need to be clonable.
+ *
+ * An undo command needs to be clonable when it may be used as a child of a
+ * command that may be merged with another command (which calls
+ * cloneChildren()).
+ */
+class ClonableUndoCommand
+{
+public:
+    virtual ~ClonableUndoCommand() = default;
+    virtual QUndoCommand *clone(QUndoCommand *parent = nullptr) const = 0;
+};
+
+bool cloneChildren(const QUndoCommand *command, QUndoCommand *parent);
+
+} // namespace Tiled
