@@ -23,8 +23,9 @@
 
 #include <QtVariantEditorFactory>
 
+class QComboBox;
+
 namespace Tiled {
-namespace Internal {
 
 class FileEdit;
 class TextPropertyEdit;
@@ -58,16 +59,16 @@ protected:
                           QWidget *parent) override;
     void disconnectPropertyManager(QtVariantPropertyManager *manager) override;
 
-private slots:
+private:
     void slotPropertyChanged(QtProperty *property, const QVariant &value);
     void slotPropertyAttributeChanged(QtProperty *property,
                                       const QString &attribute,
                                       const QVariant &value);
-    void fileEditFilePathChanged(const QString &value);
+    void fileEditFileUrlChanged(const QUrl &value);
     void textPropertyEditTextChanged(const QString &value);
+    void comboBoxPropertyEditTextChanged(const QString &value);
     void slotEditorDestroyed(QObject *object);
 
-private:
     QMap<QtProperty *, QList<FileEdit *> > mCreatedFileEdits;
     QMap<FileEdit *, QtProperty *> mFileEditToProperty;
 
@@ -76,7 +77,9 @@ private:
 
     QMap<QtProperty *, QList<TextPropertyEdit *> > mCreatedTextPropertyEdits;
     QMap<TextPropertyEdit *, QtProperty *> mTextPropertyEditToProperty;
+
+    QMap<QtProperty *, QList<QComboBox *> > mCreatedComboBoxes;
+    QMap<QComboBox *, QtProperty *> mComboBoxToProperty;
 };
 
-} // namespace Internal
 } // namespace Tiled
