@@ -26,8 +26,6 @@ namespace Tiled {
 
 class WangColor;
 
-namespace Internal {
-
 class WangColorView : public QTreeView
 {
     Q_OBJECT
@@ -40,15 +38,13 @@ protected:
     void contextMenuEvent(QContextMenuEvent *event) override;
 
 signals:
-    void wangColorColorPicked(const QColor &color, bool isEdge, int index);
+    void wangColorColorPicked(WangColor *wangColor, const QColor &color);
 
-private slots:
+private:
     void pickColor();
     void colorPicked(const QColor &color);
 
-private:
     QSharedPointer<WangColor> mClickedWangColor;
 };
 
-} // namespace Internal
 } // namespace Tiled
